<?php

require_once 'lib_gui.php';
require_once 'PHPUnit/Framework/TestCase.php';


class TestReorder extends PHPUnit_Framework_TestCase
{
	protected $fixtureGroup;
	protected $fixtureCorrect;
	public function __construct()
	{
		$this->fixtureGroup = null;
		$this->fixtureCorrect = null;
	}

	public function setup()
	{
		$this->fixtureGroup = array
		(
			'HOST_NAME' => array(),
			'SERVICE_DESCRIPTION' => array(),
		);
		$this->fixtureCorrect = array();
	}
	public function teardown()
	{
		$this->fixtureGroup = null;
		$this->fixtureCorrect = null;
	}

	public function test_reorder_hostgroups() /* {{{ */
	{
		$this->fixtureGroup = array
		(
			'HOST_NAME' => array
			(
				'share-gbg.int.op5.se',
				'switch1-gbg.int.op5.se',
				'mail.op5.se',
				'rt.op5.com',
				'backup-gbg.int.op5.se',
				'devel.int.op5.se',
				'vmware-gbg.int.op5.se',
				'w2k3std.op5.se',
			),
			'host_link' => array
			(
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
				'/monitor...assumedservicestate=-3',
			),
			'up' => Array(100, 100, 0, 100, 100, 100, 100, 0),
			'down' => Array(0, 0, 100, 0, 0, 0, 0, 100),
			'unreachable' => Array(0, 0, 0, 0, 0, 0, 0, 0),
			'undetermined' => Array(0, 0, 0, 0, 0, 0, 0, 0),
			'nr_of_items' => 8,
			'average_up' => 81.250,
			'average_down' => 18.750,
			'average_unreachable' => 0,
			'average_undetermined' => 0,
			'group_average_up' => 0,
			'group_average_down' => 100,
			'group_average_unreachable' => 0,
			'group_average_undetermined' => 0,
			'groupname' => 'Hostgroup: gbg-hosts',
		);

		# extra data that should end up
		# matching fixtureCorrect
		$this->fixtureGroup['testnumbers'] = array
		(
			4, 5, 2, 3, 0, 1, 6, 7
		);

		$this->fixtureCorrect = array
		(
			0, 1, 2, 3, 4, 5, 6, 7
		);

		reorder_by_host_and_service($this->fixtureGroup);

		for($i=0, $num_entries = count($this->fixtureGroup) ; $i<$num_entries ; $i++)
		{
			$this->assertEquals($this->fixtureGroup['testnumbers'][$i], $this->fixtureCorrect[$i]);
		}
	} /* }}} */
	public function test_reorder_services() /* {{{ */
	{
		$this->fixtureGroup = array
		(
			'HOST_NAME' => Array
			(
            	'backup-gbg.int.op5.se',
            	'devel.int.op5.se',
            	'backup-gbg.int.op5.se',
            	'devel.int.op5.se',
            	'devel.int.op5.se',
            	'backup-gbg.int.op5.se',
        	),
			'SERVICE_DESCRIPTION' => Array
			(
            	'Cron process',
            	'Cron process',
            	'Disk usage /',
            	'Disk usage /',
            	'HTTP Server',
            	'Kernel log process',
        	),
			'host_link' => Array
			(
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
	        ),
			'service_link' => Array
			(
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
            	'/monitor/op5/reports/...=-3',
        	),
			'ok' => Array(0, 0, 0, 0, 100, 0),
			'warning' => Array(0, 0, 0, 0, 0, 0),
			'unknown' => Array(100, 100, 100, 100, 0, 100),
			'critical' => Array(0, 0, 0, 0, 0, 0),
			'undetermined' => Array(0, 0, 0, 0, 0, 0),
			'nr_of_items' => 6,
			'average_ok' => 16.667,
			'average_warning' => 0,
			'average_unknown' => 83.333,
			'average_critical' => 0,
			'average_undetermined' => 0,
			'group_average_ok' => 0,
			'group_average_warning' => 0,
			'group_average_unknown' => 100,
			'group_average_critical' => 0,
			'group_average_undetermined' => 0,
			'groupname' => '',
		);

		# extra data that should end up
		# matching fixtureCorrect
		$this->fixtureGroup['testnumbers'] = array
		(
			0, 3, 1, 4, 5, 2
		);

		$this->fixtureCorrect = array
		(
			0, 1, 2, 3, 4, 5
		);

		reorder_by_host_and_service($this->fixtureGroup);

		for($i=0, $num_entries = count($this->fixtureGroup) ; $i<$num_entries ; $i++)
		{
			$this->assertEquals($this->fixtureGroup['testnumbers'][$i], $this->fixtureCorrect[$i]);
		}
	} /* }}} */
}
