<?php
require_once('sql_class.php');
$gui_db = 'monitor_gui';
$gui_user = 'monitor';
$gui_passwd = 'monitor';
$gui_host = 'localhost';
$gui_type = 'mysql';
$sqlobj = new sql_class($gui_db, $gui_user, $gui_passwd, 0, $gui_host, $gui_type);

$map_type_field = array(
	'hosts' => "host_name",
	'services' => "service_description",
	'hostgroups' => "hostgroup",
	'servicegroups' => "servicegroup"
);

$avail_setup_keys = array(
	'report_name',
	'info',
	'rpttimeperiod',
	'report_period',
	'start_time',
	'end_time',
	'report_type',
	'initialassumedhoststate',
	'initialassumedservicestate',
	'assumeinitialstates',
	'scheduleddowntimeasuptime',
	'assumestatesduringnotrunning',
	'includesoftstates'
);

/**
 * Build the web-form for the "make testcase" button.
 * This cannot be in avail_result.php where it's used, since
 * that file is include()'ed several times from send-report.php,
 * and would thus yield a "function already declared" error.
 */
function build_testcase_form($test, $prefix = '', $suffix = '')
{
	if (!is_array($test) || empty($test))
		return '';

	if (!$prefix)
		$test_buf = "\n<form action='mktest.php' method='post'>\n";
	else
		$test_buf = '';

	foreach ($test as $k => $v) {
		if (is_array($v)) {
			$test_buf .= build_testcase_form($v, $k, $suffix . '[]');
			continue;
		}
		$test_buf .= "\t<input type='hidden' value='$v' ";
		if ($prefix)
			$test_buf .= "name='test[$prefix]$suffix'";
		else
			$test_buf .= "name='test[$k]$suffix'";
		$test_buf .= " />\n";
	}
	if (!$prefix)
		$test_buf .= "<input type='submit' name='action' value='Make testcase' />" .
		"</form>\n";

	return $test_buf;
}

/**
*	@name get_host_servicegroups
*	@desc
*
*/
function get_host_servicegroups($type=false)
{
	global $mon_auth;
	if ($type === 'servicegroup')
		return $mon_auth->get_authorized_servicegroups();

	return $mon_auth->get_authorized_hostgroups();
}

/**
*	@name 	get_items
*	@desc
*
*/
function get_group_member($input=false, $type=false, $erase=true)
{
	global $mon_auth;
	if (empty($type)) {
		return false;
	}
	$return = false;
	$items = false;
	switch ($type) {
		case 'hostgroup': case 'servicegroup':
			$field_name = $type."_tmp[]";
			$empty_field = $type.'[]';
			$res = get_host_servicegroups($type);
			if (!$res) {
				return false;
			}
			foreach ($res as $name) {
				$items[] = $name;
			}
			break;
		case 'host':
			$field_name = "host_tmp[]";
			$empty_field = 'host_name[]';
			$items = $mon_auth->get_authorized_hosts();
			break;
		case 'service':
			$field_name = "service_tmp[]";
			$empty_field = 'service_description[]';
			$items = $mon_auth->get_authorized_services();
			break;
	}

	// Instantiate the xajaxResponse object
	$objResponse = new xajaxResponse();

	$objResponse->call("show_progress", "progress", 'Please wait');

	# empty both select lists before populating if it's not a saved report
	if (empty($erase)) {
		$objResponse->call("empty_list", $field_name);
		$objResponse->call("empty_list", $empty_field);
	}

	sort($items);
	$return_data = false;
	foreach ($items as $k => $item) {
		$return_data[] = array('optionValue' => $item, 'optionText' => $item);
	}
	if (!function_exists('json_encode')) {
		$json = new JSON();
		$json_val = $json->encode($return_data);
	} else {
		$json_val = json_encode($return_data);
	}

	# pass the JSON data to javascript to build the HTML select options
	$objResponse->call("populate_options", $field_name, $empty_field, $json_val);
	#$objResponse->call("setup_hide_content", "progress");

	//return the  xajaxResponse object
	return $objResponse;
}

/**
*	@name calculate_time
*	@desc
*
*/
function calculate_time($report_period='', $start_date=false, $end_date=false)
{
	$year_now 	= date('Y', time());
	$month_now 	= date('m', time());
	$day_now	= date('d', time());
	$week_now 	= date('W', time());
	$weekday_now = date('w', time())-1;
	$time_start	= false;
	$time_end	= false;
	$now = time();

	switch ($report_period) {
		case 'today':
			$time_start = mktime(0, 0, 0, $month_now, $day_now, $year_now);
			$time_end 	= time();
			break;
		case 'last24hours':
			$time_start = mktime(date('H', time()), date('i', time()), date('s', time()), $month_now, $day_now -1, $year_now);
			$time_end 	= time();
			break;
		case 'yesterday':
			$time_start = mktime(0, 0, 0, $month_now, $day_now -1, $year_now);
			$time_end 	= mktime(0, 0, 0, $month_now, $day_now, $year_now);
			break;
		case 'thisweek':
			$time_start = strtotime('today - '.$weekday_now.' days');
			$time_end 	= time();
			break;
		case 'last7days':
			$time_start	= strtotime('now - 7 days');
			$time_end	= time();
			break;
		case 'lastweek':
			$time_start = strtotime('midnight last monday -7 days');
			$time_end	= strtotime('midnight last monday');
			break;
		case 'thismonth':
			$time_start = strtotime('midnight '.$year_now.'-'.$month_now.'-01');
			$time_end	= time();
			break;
		case 'last31days':
			$time_start = strtotime('now - 31 days');
			$time_end	= time();
			break;
		case 'lastmonth':
			$time_start = strtotime('midnight '.$year_now.'-'.$month_now.'-01 -1 month');
			$time_end	= strtotime('midnight '.$year_now.'-'.$month_now.'-01');
			break;
		case 'thisyear':
			$time_start = strtotime('midnight '.$year_now.'-01-01');
			$time_end	= time();
			break;
		case 'lastyear':
			$time_start = strtotime('midnight '.$year_now.'-01-01 -1 year');
			$time_end	= strtotime('midnight '.$year_now.'-01-01');
			break;
		case 'last12months':
			$time_start	= strtotime('midnight '.$year_now.'-'.$month_now.'-01 -12 months');
			$time_end	= strtotime('midnight '.$year_now.'-'.$month_now.'-01');
			break;
		case 'last3months':
			$time_start	= strtotime('midnight '.$year_now.'-'.$month_now.'-01 -3 months');
			$time_end	= strtotime('midnight '.$year_now.'-'.$month_now.'-01');
			break;
		case 'last6months':
			$time_start	= strtotime('midnight '.$year_now.'-'.$month_now.'-01 -6 months');
			$time_end	= strtotime('midnight '.$year_now.'-'.$month_now.'-01');
			break;
		case 'lastquarter':
			$t = getdate();
			if($t['mon'] <= 3){
				$lqstart = ($t['year']-1)."-10-01";
				$lqend = ($t['year']-1)."-12-31";
			} elseif ($t['mon'] <= 6) {
				$lqstart = $t['year']."-01-01";
				$lqend = $t['year']."-03-31";
			} elseif ($t['mon'] <= 9){
				$lqstart = $t['year']."-04-01";
				$lqend = $t['year']."-06-30";
			} else {
				$lqstart = $t['year']."-07-01";
				$lqend = $t['year']."-09-30";
			}
			$time_start = strtotime($lqstart);
			$time_end = strtotime($lqend);
			break;
		case 'custom':
			$time_start = is_numeric($start_date) ? $start_date : strtotime($start_date);
			$time_end = is_numeric($end_date) ? $end_date : strtotime($end_date);
			break;
		default:
			if (empty($start_date) || !is_numeric($start_date)) {
				# unknown report_period and no fallback available, so
				# default to last 24 hours, like old cgi's used to do
				return calculate_time('last24hours');
			}

			# $start_date looks sensible, so try some DWIM'ery
			$time_start = $start_date;

			# if no end-date, or end_date is bad, use $now
			if (empty($end_date) || !is_numeric($end_date)
			    || $end_date > $now || $end_date < $time_start)
			{
				$end_date = $now;
			}

			$time_end = $end_date;
	}

	if($time_start > $now)
		$time_start = $now;

	if($time_end > $now)
		$time_end = $now;

	return array($time_start, $time_end);
}

/**
 * @name 	time_to_string
 * @desc	Convert a given nr of sec to string,
 * 			day, hour, minute, second
 *
 * @param 	int $t seconds
 * @return 	str
 */
function time_to_string($t=0)
{
	$neg = false;
	if (!$t) return "0d 0h 0m 0s";
	if ($t < 0) {
		$neg 	= 1;
		$t 		= 0 - $t;
	}

	$days 	= $t / 86400;
	$days 	= floor($days);
	$hrs 	= ($t / 3600) % 24;
	$mins 	= ($t / 60) % 60;
	$secs 	= $t % 60;

	$timestring = "";
	if ($neg) $timestring .= "negative ";
	if ($days) {
		$timestring .= $days."d";
		if ($hrs || $mins || $secs) $timestring .= " ";
	}
	if ($hrs) {
		$timestring .= $hrs."h";
		if ($mins && $secs) $timestring .= " ";
	}
	if ($mins) {
		$timestring .= $mins."m";
		if ($mins && $secs) $timestring .= " ";
	}
	if ($secs) $timestring .= $secs."s";
	return $timestring;
}

/**
*	@name print_state_breakdowns
*	@desc
*
*/
function print_state_breakdowns($source=false, $values=false, $type="hosts")
{
	if (empty($values)) {
		return false;
	}
	$tot_time = 0;
	$tot_time_perc = 0;
	$tot_time_known_perc = 0;
	if ($type=='hosts' || $type=='hostgroups') {
		$tot_time = $values['KNOWN_TIME_UP'] +
			$values['KNOWN_TIME_DOWN'] +
			$values['KNOWN_TIME_UNREACHABLE'] +
			$values['TOTAL_TIME_UNDETERMINED'];
		$tot_time_perc = $values['PERCENT_KNOWN_TIME_UP'] +
			$values['PERCENT_KNOWN_TIME_DOWN'] +
			$values['PERCENT_KNOWN_TIME_UNREACHABLE'] +
			$values['PERCENT_TIME_UNDETERMINED_NOT_RUNNING'] +
			$values['PERCENT_TIME_UNDETERMINED_NO_DATA'];
		/*
		$tot_time_known_perc = $values['PERCENT_KNOWN_TIME_UP'] +
			$values['PERCENT_KNOWN_TIME_DOWN'] +
			$values['PERCENT_KNOWN_TIME_UNREACHABLE'];
		*/
		$var_types = array('UP', 'DOWN', 'UNREACHABLE');
	} else {
		$tot_time = $values['KNOWN_TIME_OK'] +
			$values['KNOWN_TIME_WARNING'] +
			$values['KNOWN_TIME_UNKNOWN'] +
			$values['KNOWN_TIME_CRITICAL'] +
			$values['TOTAL_TIME_UNDETERMINED'];
		$tot_time_perc = $values['PERCENT_KNOWN_TIME_OK'] +
			$values['PERCENT_KNOWN_TIME_WARNING'] +
			$values['PERCENT_KNOWN_TIME_UNKNOWN'] +
			$values['PERCENT_KNOWN_TIME_CRITICAL'] +
			$values['PERCENT_TIME_UNDETERMINED_NOT_RUNNING'] +
			$values['PERCENT_TIME_UNDETERMINED_NO_DATA'];
		/*
		$tot_time_known_perc = $values['PERCENT_KNOWN_TIME_OK'] +
			$values['PERCENT_KNOWN_TIME_WARNING'] +
			$values['PERCENT_KNOWN_TIME_UNKNOWN'] +
			$values['PERCENT_KNOWN_TIME_CRITICAL'] +
			$values['PERCENT_KNOWN_TIME_UNREACHABLE'];
		*/
		$var_types = array('OK', 'WARNING', 'UNKNOWN', 'CRITICAL');
	}

	return array('tot_time' => $tot_time, 'tot_time_perc' => $tot_time_perc, 'var_types' => $var_types, 'values' => $values);
	// 'tot_time_known_perc' => $tot_time_known_perc,
}

function is_proper_report_item($k, $data)
{
	if (is_array($data) && !empty($data['states']) && is_array($data['states']))
		return true;

	return false;
}

/**
*	@name get_multiple_state_info
*	@desc
*
* 	@param array  $data_arr report source data, generated by report_class:get_uptime()
* 	@param string $type The report subtype. Can be 'host' or 'service'.
* 	@param string $get_vars query string containing values of options for the report<br>
*       Contains: report_period, rpttimeperiod, scheduleddowntimeasuptime, assumestatesduringnotrunning, includesoftstates, assumeinitialstates, initialassumedhoststate, initialassumedservicestate)
* 	@param int $start_time Start timestamp for the report.
* 	@param int $end_time End timestamp for the report.
*
* 	@return	array report info divided by states
*
*/
function get_multiple_state_info(&$data_arr, $type, $get_vars, $start_time, $end_time)
{
	$start_time = date("Y-m-d H:i", $start_time);
	$end_time = date("Y-m-d H:i", $end_time);
	$prev_host = '';
	$php_self = $_SERVER['PHP_SELF'];
#	if (array_key_exists('groupname', $data_arr) && $data_arr['groupname']!='')
		//echo "<span>".ucfirst($type).'group: '.$data_arr['groupname']."</span>";
	if (array_key_exists('states', $data_arr) && !empty($data_arr['states']))
		$group_averages = $data_arr['states'];

	$return = array();
	$cnt = 0;
	if ($type=='service') {
		$sum_ok = $sum_warning = $sum_unknown = $sum_critical = $sum_undetermined = 0;
		foreach ($data_arr as $k => $data) {
			if (!is_proper_report_item($k, $data))
				continue;

			$host_name = $data['states']['HOST_NAME'];
			$service_description = $data['states']['SERVICE_DESCRIPTION'];

			$return['host_link'][] = $php_self . "?host_name[]=". $host_name . "&report_type=hosts" . '&start_time=' . $start_time . '&end_time=' . $end_time . '&origin=print_multiple&new_avail_report_setup=1'.$get_vars;
			$return['service_link'][] = $php_self . "?host_name[]=". $host_name . '&service_description[]=' . "$host_name;$service_description" . '&report_type=services&start_time=' . $start_time . '&end_time=' . $end_time . '&origin=print_multiple&new_avail_report_setup=1'.$get_vars;

			$return['HOST_NAME'][] 				= $host_name;
			$return['SERVICE_DESCRIPTION'][] 	= $service_description;
			$return['ok'][] 			= $data['states']['PERCENT_KNOWN_TIME_OK'];
			$return['warning'][] 		= $data['states']['PERCENT_KNOWN_TIME_WARNING'];
			$return['unknown'][] 		= $data['states']['PERCENT_KNOWN_TIME_UNKNOWN'];
			$return['critical'][] 		= $data['states']['PERCENT_KNOWN_TIME_CRITICAL'];
			$return['undetermined'][] 	= $data['states']['PERCENT_TOTAL_TIME_UNDETERMINED'];

			$prev_host = $host_name;
			$sum_ok += $data['states']['PERCENT_KNOWN_TIME_OK'];
			$sum_warning += $data['states']['PERCENT_KNOWN_TIME_WARNING'];
			$sum_unknown += $data['states']['PERCENT_KNOWN_TIME_UNKNOWN'];
			$sum_critical += $data['states']['PERCENT_KNOWN_TIME_CRITICAL'];
			$sum_undetermined += $data['states']['PERCENT_TOTAL_TIME_UNDETERMINED'];
			$cnt++;
		}
		$return['nr_of_items'] = $cnt;
		$return['average_ok'] = $sum_ok!=0 ? format_report_value($sum_ok/$cnt) : '0';
		$return['average_warning'] = $sum_warning!=0 ? format_report_value($sum_warning/$cnt) : '0';
		$return['average_unknown'] = $sum_unknown!=0 ? format_report_value($sum_unknown/$cnt) : '0';
		$return['average_critical'] = $sum_critical!=0 ? format_report_value($sum_critical/$cnt) : '0';
		$return['average_undetermined'] = $sum_undetermined!=0 ? format_report_value($sum_undetermined/$cnt) : '0';
		$return['group_average_ok'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_OK']);
		$return['group_average_warning'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_WARNING']);
		$return['group_average_unknown'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_UNKNOWN']);
		$return['group_average_critical'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_CRITICAL']);
		$return['group_average_undetermined'] = format_report_value($group_averages['PERCENT_TOTAL_TIME_UNDETERMINED']);
		$return['groupname'] = $data_arr['groupname']!='' ? 'Servicegroup: '.$data_arr['groupname'] : false;
		$return[';testcase;'] = $data_arr[';testcase;'];
	} else {
		// host
		$sum_up = $sum_down = $sum_unreachable = $sum_undetermined = 0;
		foreach ($data_arr as $k => $data) {
			if (!is_proper_report_item($k, $data))
				continue;
			$host_name = $data['states']['HOST_NAME'];
			$return['host_link'][] = $php_self . "?host_name[]=". $host_name. "&report_type=hosts" .
			'&start_time=' . $start_time . '&end_time=' . $end_time .'&origin=print_multiple'.$get_vars;
			$return['HOST_NAME'][] 		= $host_name;
			$return['up'][] 			= $data['states']['PERCENT_KNOWN_TIME_UP'];
			$return['down'][] 			= $data['states']['PERCENT_KNOWN_TIME_DOWN'];
			$return['unreachable'][]	= $data['states']['PERCENT_KNOWN_TIME_UNREACHABLE'];
			$return['undetermined'][]	= $data['states']['PERCENT_TOTAL_TIME_UNDETERMINED'];

			$sum_up += $data['states']['PERCENT_KNOWN_TIME_UP'];
			$sum_down += $data['states']['PERCENT_KNOWN_TIME_DOWN'];
			$sum_unreachable += $data['states']['PERCENT_KNOWN_TIME_UNREACHABLE'];
			$sum_undetermined += $data['states']['PERCENT_TOTAL_TIME_UNDETERMINED'];
			$cnt++;
		}
		$return['nr_of_items'] = $cnt;
		$return['average_up'] = $sum_up!=0 ? format_report_value($sum_up/$cnt) : '0';
		$return['average_down'] =  $sum_down!=0 ? format_report_value($sum_down/$cnt) : '0';
		$return['average_unreachable'] = $sum_unreachable!=0 ? format_report_value($sum_unreachable/$cnt) : '0';
		$return['average_undetermined'] = $sum_undetermined!=0 ? format_report_value($sum_undetermined/$cnt) : '0';

		$return['group_average_up'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_UP']);
		$return['group_average_down'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_DOWN']);
		$return['group_average_unreachable'] = format_report_value($group_averages['PERCENT_KNOWN_TIME_UNREACHABLE']);
		$return['group_average_undetermined'] = format_report_value($group_averages['PERCENT_TOTAL_TIME_UNDETERMINED']);
		$return['groupname'] = $data_arr['groupname']!='' ? 'Hostgroup: '.$data_arr['groupname'] : false;
		$return[';testcase;'] = $data_arr[';testcase;'];
	}
#		echo "<pre>";
#		print_r($return);
#		echo "</pre>";
	return $return;
}

/**
*	@name 	format_report_value
*	@desc 	Format report value output
*
*/
function format_report_value($val)
{
	$return = 0;
	if ($val == '0.000' || $val == '100.000')
		$return = number_format($val, 0);
	else
		$return = number_format($val, 3);

	return $return;
}

/**
*	@name 	get_csv_fields
*	@desc 	Returns the fields needed for csv output.
* 			The order of the fields are the same as in avail.cgi
*
*/
function get_csv_fields($type=false)
{
	$fields['host'] = array(
		'HOST_NAME',
		'TIME_UP_SCHEDULED',
		'PERCENT_TIME_UP_SCHEDULED',
		'PERCENT_KNOWN_TIME_UP_SCHEDULED',
		'TIME_UP_UNSCHEDULED',
		'PERCENT_TIME_UP_UNSCHEDULED',
		'PERCENT_KNOWN_TIME_UP_UNSCHEDULED',
		'TOTAL_TIME_UP',
		'PERCENT_TOTAL_TIME_UP',
		'PERCENT_KNOWN_TIME_UP',
		'TIME_DOWN_SCHEDULED',
		'PERCENT_TIME_DOWN_SCHEDULED',
		'PERCENT_KNOWN_TIME_DOWN_SCHEDULED',
		'TIME_DOWN_UNSCHEDULED',
		'PERCENT_TIME_DOWN_UNSCHEDULED',
		'PERCENT_KNOWN_TIME_DOWN_UNSCHEDULED',
		'TOTAL_TIME_DOWN',
		'PERCENT_TOTAL_TIME_DOWN',
		'PERCENT_KNOWN_TIME_DOWN',
		'TIME_UNREACHABLE_SCHEDULED',
		'PERCENT_TIME_UNREACHABLE_SCHEDULED',
		'PERCENT_KNOWN_TIME_UNREACHABLE_SCHEDULED',
		'TIME_UNREACHABLE_UNSCHEDULED',
		'PERCENT_TIME_UNREACHABLE_UNSCHEDULED',
		'PERCENT_KNOWN_TIME_UNREACHABLE_UNSCHEDULED',
		'TOTAL_TIME_UNREACHABLE',
		'PERCENT_TOTAL_TIME_UNREACHABLE',
		'PERCENT_KNOWN_TIME_UNREACHABLE',
		'TIME_UNDETERMINED_NOT_RUNNING',
		'PERCENT_TIME_UNDETERMINED_NOT_RUNNING',
		'TIME_UNDETERMINED_NO_DATA',
		'PERCENT_TIME_UNDETERMINED_NO_DATA',
		'TOTAL_TIME_UNDETERMINED',
		'PERCENT_TOTAL_TIME_UNDETERMINED'
	);
	$fields['service'] = array(
	'HOST_NAME',
	'SERVICE_DESCRIPTION',
	'TIME_OK_SCHEDULED',
	'PERCENT_TIME_OK_SCHEDULED',
	'PERCENT_KNOWN_TIME_OK_SCHEDULED',
	'TIME_OK_UNSCHEDULED',
	'PERCENT_TIME_OK_UNSCHEDULED',
	'PERCENT_KNOWN_TIME_OK_UNSCHEDULED',
	'TOTAL_TIME_OK',
	'PERCENT_TOTAL_TIME_OK',
	'PERCENT_KNOWN_TIME_OK',
	'TIME_WARNING_SCHEDULED',
	'PERCENT_TIME_WARNING_SCHEDULED',
	'PERCENT_KNOWN_TIME_WARNING_SCHEDULED',
	'TIME_WARNING_UNSCHEDULED',
	'PERCENT_TIME_WARNING_UNSCHEDULED',
	'PERCENT_KNOWN_TIME_WARNING_UNSCHEDULED',
	'TOTAL_TIME_WARNING',
	'PERCENT_TOTAL_TIME_WARNING',
	'PERCENT_KNOWN_TIME_WARNING',
	'TIME_UNKNOWN_SCHEDULED',
	'PERCENT_TIME_UNKNOWN_SCHEDULED',
	'PERCENT_KNOWN_TIME_UNKNOWN_SCHEDULED',
	'TIME_UNKNOWN_UNSCHEDULED',
	'PERCENT_TIME_UNKNOWN_UNSCHEDULED',
	'PERCENT_KNOWN_TIME_UNKNOWN_UNSCHEDULED',
	'TOTAL_TIME_UNKNOWN',
	'PERCENT_TOTAL_TIME_UNKNOWN',
	'PERCENT_KNOWN_TIME_UNKNOWN',
	'TIME_CRITICAL_SCHEDULED',
	'PERCENT_TIME_CRITICAL_SCHEDULED',
	'PERCENT_KNOWN_TIME_CRITICAL_SCHEDULED',
	'TIME_CRITICAL_UNSCHEDULED',
	'PERCENT_TIME_CRITICAL_UNSCHEDULED',
	'PERCENT_KNOWN_TIME_CRITICAL_UNSCHEDULED',
	'TOTAL_TIME_CRITICAL',
	'PERCENT_TOTAL_TIME_CRITICAL',
	'PERCENT_KNOWN_TIME_CRITICAL',
	'TIME_UNDETERMINED_NOT_RUNNING',
	'PERCENT_TIME_UNDETERMINED_NOT_RUNNING',
	'TIME_UNDETERMINED_NO_DATA',
	'PERCENT_TIME_UNDETERMINED_NO_DATA',
	'TOTAL_TIME_UNDETERMINED',
	'PERCENT_TOTAL_TIME_UNDETERMINED'
	);
	return $fields[$type];
}

/**
*	@name 	csv_content
*	@desc 	Returns the csv value string to be printed for
* 			the selected type (host/service)
*
*/
function csv_content(&$states=false, $type='host')
{
	if (!$states) {
		return false;
	}
	$csv = false;

	$fields = get_csv_fields($type);

	foreach ($fields as $field_name) {
		if ($field_name == 'HOST_NAME' || $field_name == 'SERVICE_DESCRIPTION') {
			$csv[] = '"' . $states[$field_name] . '"';
		} else {
			$csv[] = strstr($field_name, 'PERCENT') ? format_report_value($states[$field_name]).'%' : $states[$field_name];
		}
	}
	//return implode(', ', $csv)."\n";
	return implode(', ', $csv);
}

/**
*	@name 	csv_header
*	@desc 	Get the csv header line
*
*/
function csv_header($type=false)
{
	$fields = get_csv_fields($type);
	$csv = implode(', ', $fields);
	return $csv."\n";
}


/**
*	@name 	img_filename_convert
*	@desc 	Convert nasty chars before creating report image file
*
*/
function img_filename_convert($filename=false)
{
	$filename = trim($filename);
	$filename = str_replace('/', '-', $filename);
	$filename = str_replace(' ', '_', $filename);
	$filename = str_replace(';', '_', $filename);

	return $filename;
}

/**
*	@name get_services_for_host
*	@desc
*
*/
function get_services_for_host($host_name=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	$host_name = trim($host_name);
	if (empty($host_name)) {
		return false;
	}
	$sql = "SELECT s.service_description FROM ".GUI_DB.".host h, ".GUI_DB.".service s " .
	" WHERE h.host_name='".$host_name."' AND s.host_name=h.id ORDER BY s.service_description";
	if (!$res = $dba->execute($sql)) {
		if (!empty($dba->error))
			__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
		return false;
	}
	return $res;
}

/**
*	@name print_states_for_services
*	@desc
*
*/
function print_states_for_services($host_name=false, $start_date=false, $end_date=false)
{
	global $options, $dep_vars, $_REQUEST, $err_msg;
	$host_name = trim($host_name);
	if (empty($host_name)) {
		return false;
	}
	$res = get_services_for_host($host_name);
	$service_arr = array();
	if (!empty($res)) {
		$report_class = new report_class();
		foreach ($res as $row)
			$service_arr[] = $host_name.";".$row['service_description'];

		foreach ($options as $var => $new_var) {
			if (!$report_class->set_option($new_var, get_array_var($_REQUEST, $var))) {
				$err_msg .= "Could not set option '$new_var' to '".convert_yesno_int(get_array_var($_REQUEST, $var))."'<br />";
			}
		}

		foreach ($dep_vars as $check => $set)
			if (isset($_REQUEST[$check]) && !empty($_REQUEST[$check]))
				foreach ($set as $dep => $key)
					if (!$report_class->set_option($key, $_REQUEST[$dep]))
						$err_msg .= "Could not set option '".$key."' to '".$_REQUEST[$dep]."'<br />";

		$report_class->set_option('host_name', $host_name);
		$report_class->set_option('service_description', $service_arr);

		$data_arr = $report_class->get_uptime($host_name, $service_arr, $start_date, $end_date, false, false);
		return $data_arr;
	}
	return false;
}

function check_report_type($report_type=false, $in_host=false, $in_service=false, $servicegroup=false, $hostgroup=false)
{
	if (empty($report_type)) {
		if (!empty($in_host)) {
			if (!empty($in_service)) {
				$report_type = 'services';
			} else {
				$report_type = 'hosts';
			}
		} else {
			if (!empty($servicegroup)) {
				$report_type = 'servicegroups';
			} else {
				if (!empty($hostgroup)) {
					$report_type = 'hostgroups';
				}
			}
		}
	}
	return $report_type;
}

/**
*	@name 	get_reporting_periods
*	@desc 	Fetch and print information on saved timperiods
*
*/
function get_reporting_periods()
{
	global $dba;

	$return = "";
	$sql = "SELECT timeperiod_name FROM timeperiod ORDER BY timeperiod_name";
	if (!$res = $dba->execute($sql)) {
		if (!empty($dba->error))
			__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
		return $dba->error;
	}
	foreach ($res as $row) {
		echo '<option value="'.$row['timeperiod_name'].'">'.$row['timeperiod_name'].'</option>';
	}
	return $return;
}

/**
 * Fetch host alias information from database
 *
 * @param str $host_name
 */
function get_host_alias($host_name=false)
{
	if (empty($host_name))
		return false;

	global $dba;
	$host_name = trim($host_name);
	$sql = "SELECT alias FROM host WHERE host_name='".$dba->escapeStr($host_name)."'";
	if (!$res = $dba->query($sql)) {
		if (!empty($dba->error))
			__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
		return $dba->error;
	}
	return $dba->getValue();
}

/**
 * Expands a series of groupnames (host or service) into its member objects, and calculate uptime for each
 *
 * @param array $arr List of groups
 * @param string $type The type of objects in $arr. Valid values are "hostgroup" or "servicegroup".
 * @param mixed $start_date datetime or unix timestamp
 * @param mixed $end_date datetime or unix timestamp
 * @global array Report options
 * @global array Dependent report options
 * @global array
 * @global string Error log.
 * @return array Calculated uptimes.
 */
function expand_group_request($arr=false, $type='hostgroup', $start_date=false, $end_date=false)
{
	global $options, $dep_vars, $_REQUEST, $err_msg;
	if (empty($arr))
		return false;
	if ($type!='hostgroup' && $type!='servicegroup')
		return false;
	$hostgroup = false;
	$servicegroup = false;
	$data_arr = false;
	foreach ($arr as $$type) {
		$rpt_class = new report_class();
		foreach ($options as $var => $new_var) {
			if (!$rpt_class->set_option($new_var, get_array_var($_REQUEST, $var))) {
				$err_msg .= "Could not set option '$new_var' to '".convert_yesno_int(get_array_var($_REQUEST, $var))."'<br />";
			}
		}
		foreach ($dep_vars as $check => $set)
			if (isset($_REQUEST[$check]) && !empty($_REQUEST[$check]))
				foreach ($set as $dep => $key)
					if (!$rpt_class->set_option($key, $_REQUEST[$dep]))
						$err_msg .= "Could not set option '".$key."' to '".$_REQUEST[$dep]."'<br />";

		$rpt_class->set_option(substr($type, 0, strlen($type)).'_name', $$type);
		$data_arr[] = $rpt_class->get_uptime(false, false, $start_date, $end_date, $hostgroup, $servicegroup);
	}

	return $data_arr;
}

/**
*	@name 	convert_yesno_int
*	@desc 	Convert between yes/no and 1/0
* 	@param 	mixed val, value to be converted
* 	@param 	bool use_int, to indicate if we should use
* 			1/0 instead of yes/no
* 	@return mixed str/int
*
*/
function convert_yesno_int($val, $use_int=true)
{
	$return = false;
	if ($use_int) {
		// This is the way that we normally do things
		switch (strtolower($val)) {
			case 'yes':
				$return = 1;
				break;
			case 'no':
				$return = 0;
				break;
			default:
				$return = $val;
		}
	} else {
		// This is the old way, using yes/no values
		switch ($val) {
			case 1:
				$return = 'yes';
				break;
			case 0:
				$return = 'no';
				break;
			default:
				$return = $val;
		}
	}
	return $return;
}

/**
 * @desc	Convert assumed states between old cgi's and
 * 			new avail_result.php.
 *
 * @param 	mixed $val, Value to be converted
 * @param 	str $type, host/service
 * @param 	bool $to_new,
 * 				true => return NEW values,
 * 				false => return OLD values
 * @return str The converted state. It is important this number is string since report_class::set_option expects this
 */
function convert_assumed_state($val=false, $type='host', $to_new=true)
{
	$arr = false;
	$retval = false;
	// new value => value used by cgi's
	$host_states = array(
		-1 => -1,
		-2 => 0,
		-3 => 0,
		0 => 3,
		1 => 4,
		2 => 5
	);

	$service_states = array(
		-1 => -1,
		-2 => 0,
		-3 => 0,
		0 => 6,
		1 => 8,
		2 => 9,
		3 => 7
	);

	switch ($type) {
		case 'host':
			$arr = $host_states;
			break;
		case 'service':
			$arr = $service_states;
			break;
		default:
			return (string)$val;
	}

	if ($to_new === false) {
		if (array_key_exists($val, $arr)) {
			return (string)$arr[$val];
		}
		// unable to convert...
		return (string)$val;
	} else {
		// convert the other way around
		// ie, return key corresponding to value
		$retval = array_search($val, $arr);
		return (string)($retval !== false ? $retval : $val);
	}
}

function get_saved_reports()
{
	global $sqlobj, $mon_auth;
	$dba = $sqlobj;
	if (!$dba) {
		$sql_class = new sql_class();
		$dba = $sql_class->db;
	}
	$sql = "SELECT id, report_name FROM avail_config ";
	if (!$mon_auth->view_hosts_root) {
		$sql .= "WHERE user='".get_array_var($_SERVER, 'REMOTE_USER')."' OR user=''";
	}
	$sql .= " ORDER BY report_name";

	$return = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $return;
}

function edit_report_info($avail_id, $avail_options, $objects)
{
	global $sqlobj;
	$dba = $sqlobj;
	$update = false;
	if (empty($avail_options) || empty($objects)) {
		return false;
	}

	// INSERT or UPDATE?
	if ($avail_id) $update = true;
	else
	{
		$sql = "SELECT id FROM avail_config WHERE report_name = '".$avail_options['report_name']."'";
		$res = $dba->execute($sql);
		if(count($res) > 0)
		{
			$avail_id = $res[0]["id"];
			$update = true;
		}
	}
	if (!$update) {
		$sql = "INSERT INTO avail_config (user, ".implode(', ', array_keys($avail_options)).") VALUES('".get_array_var($_SERVER, 'REMOTE_USER')."', '".implode('\',\'', array_values($avail_options))."')";
	} else {
		$sql = "UPDATE avail_config SET ";
		foreach ($avail_options as $key => $value) {
			$a_sql[] = $key." = '".$value."' ";
		}
		$sql .= implode(', ', $a_sql);
		$sql .= ", updated = now()";
		$sql .= " WHERE id=".$avail_id;
	}
	#echo $sql;
	$dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	// query OK, continue with objects
	if (!$update) $avail_id = (int)$dba->getInsertId();

	// insert/update avail_config_objects
	if (!save_avail_config_objects($avail_id, $objects)) {
		return false;
	}
	return $avail_id;
}

/**
*	@name 	save_avail_config_objects
*	@desc 	save information on what objects are related to this
* 			report - hosts/services/-groups
* 			Stores the name of the objects
*/
function save_avail_config_objects($avail_id=false, $objects=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	if (empty($objects) || empty($avail_id)) return false;

	// remove old records (if any)
	$sql = "DELETE FROM avail_config_objects WHERE avail_id=".$avail_id;
	$dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	unset($sql);

	$_sql = "INSERT INTO avail_config_objects (avail_id, name) VALUES(";
	foreach ($objects as $item) {
		$query = $_sql.(int)$avail_id.", '".$item."')";
		$dba->execute($query);
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			return false;
		}
	}
	return true;
}

/**
*	@name 	delete_report
*	@desc 	Delete info on a saved report
* 	@param  int $avail_id
*
*/
function delete_report($avail_id)
{
	global $sqlobj;
	$dba = $sqlobj;
	if (empty($avail_id)) return false;
	$err = false;
	$sql[] = "DELETE FROM avail_config_objects WHERE avail_id=".$avail_id;
	$sql[] = "DELETE FROM avail_config WHERE id=".$avail_id;
	foreach ($sql as $query) {
		$dba->execute($query);
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			$err++;
		}
	}
	if ($err !== false) {
		return false;
	}
	delete_scheduled_report($avail_id);
	return true;
}

function delete_scheduled_report($id=false, $identifier='avail')
{
	global $dba;
	$id = (int)$id;
	if (!$id) return false;
	$res = get_scheduled_reports($identifier);
	if (empty($res)) return false;
	$scheduled_id = false;
	foreach ($res as $row) {
		if ($row['report_id'] == $id) {
			$scheduled_id = $row['id'];
			break;
		}
	}
	if (!empty($scheduled_id)) {
		$sql = "DELETE FROM auto_reports_scheduled WHERE id=".$scheduled_id;
		$dba->execute($sql);
		return true;
	} else {
		return false;
	}
}

/**
 * @name 	get_all_avail_info_names
 * @desc   	Fetches all info names from avail_config
 * @return array all names
 *
 */
function get_all_avail_config_names()
{
	global $sqlobj;
	$dba = $sqlobj;
	$sql = "SELECT report_name FROM avail_config WHERE user='".get_array_var($_SERVER, 'REMOTE_USER')."'";
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	$names = array();
	foreach($res as $row)
	{
		$names[] = $row['report_name'];
	}
	return $names;
}

/**
*	@name 	get_report_info
*	@desc 	Fetch info on single saved report
*
*/
function get_report_info($id=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	if (empty($id)) return false;

	$sql = "SELECT * FROM avail_config WHERE id=".(int)$id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	$return = $res[0];
	$object_info = get_avail_config_objects($id);
	$objects = false;
	if ($object_info) {
		foreach ($object_info as $row) {
			$objects[] = $row['name'];
		}
	}
	$return['objects'] = $objects;
	return $return;
}

/**
*	@name get_avail_config_objects
*
*/
function get_avail_config_objects($avail_id=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	if (empty($avail_id)) {
		return false;
	}
	$sql = "SELECT * FROM avail_config_objects WHERE avail_id=".(int)$avail_id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $res;
}


/**
 * Fetches all scheduled reports of current report type (avail/sla)
 *
 * @param str $identifier {avail, sla}
 * @return res
 */
function get_scheduled_reports($identifier='avail')
{
	global $sqlobj;
	$dba = $sqlobj;
	$identifier = trim($identifier);
	$sql = "SELECT
			ars.*,
			arp.periodname
		FROM
			monitor_gui.auto_reports_scheduled ars,
			monitor_gui.auto_report_types art,
			monitor_gui.auto_reports_periods arp
		WHERE
			art.identifier='".$identifier."' AND
			ars.report_type_id=art.id AND
			arp.id=ars.period_id
		ORDER BY
			ars.id";
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $res;
}


/**
 * Checks if current saved report also is scheduled
 * in autoreports
 *
 * @param int $id
 * @param str $identifier {avail, sla}
 * @return mixed array or false on error
 */
function report_is_scheduled($id=false, $identifier='avail')
{
	$id = (int)$id;
	if (!$id) return false;
	$res = get_scheduled_reports($identifier);
	if (empty($res)) {
		return false;
	}
	$return = false;
	foreach ($res as $row) {
		if ($row['report_id'] == $id) {
			$return[] = $row;
		}
	}
	return $return;
}

function get_available_autoreport_periods()
{
	global $sqlobj;
	$dba = $sqlobj;
	$sql = "SELECT * from monitor_gui.auto_reports_periods";
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $res;
}

function fetch_scheduled_field_value($type=false, $id=false, $elem_id=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	$id = (int)$id;
	$type = trim($type);
	$elem_id = trim($elem_id);
	if (empty($type) || empty($id) || empty($elem_id)) return false;
	$objResponse = new xajaxResponse();
	$sql = "SELECT * FROM monitor_gui.auto_reports_scheduled WHERE id=".$id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		// FIXME: inform user of error - print message in some available message box
		return mysql_error();
	}
	$objResponse->call("show_progress", "progress", 'Please wait...');
	$row = $res[0];
	$objResponse->assign($elem_id,"innerHTML", $row[$type]);
	$objResponse->call('setup_hide_content', 'progress');
	return $objResponse;
}

/**
 * Delete the schedule for a report in
 * monitor_gui database
 *
 * @param 	int $id in autoreports_scheduled
 * @param 	str $context enables us to take different actions
 * 			depending on where it is called from
 * @return 	ajax output
 */
function delete_schedule_ajax($id=false, $context=false)
{
	global $sqlobj;
	$dba = $sqlobj;
	$id = (int)$id;
	$objResponse = new xajaxResponse();
	$objResponse->call('alert', 'INSIDE');
	return $objResponse;
	$objResponse->call("show_progress", "progress", 'Please wait');
	if (!$id) {
		#$objResponse->call("show_progress", "progress", 'Please wait');
		$objResponse->assign("err_msg","innerHTML", "Missing ID so nothing to delete");
		return $objResponse;
	}
	$sql = "DELETE FROM monitor_gui.auto_reports_scheduled WHERE id=".$id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		$objResponse->assign("err_msg","innerHTML", "Error when trying to delete scheduled report with ID $id: " . mysql_error());
		return $objResponse;
	}
	$objResponse->call('hide_progress');
	#$objResponse->assign("err_msg","innerHTML", "DELETED");
	switch ($context) {
		case 'setup':
			$objResponse->call('remove_deleted_rows', $id);
			break;
		case 'edit':
			$objResponse->call('remove_schedule', $id);
			break;
	}
	return $objResponse;
}


/**
 * @name reorder_by_host_and_service
 * @desc Re-order alphabetically a group to
 * 1) sort by host name
 * 2) sort by service description
 * A group here refers to the return value given by a call to get_multiple_state_info().
 * @param array &$group Return parameter.
 */
function reorder_by_host_and_service(&$group)
{
	global $report_type;

	$testcase = isset($group[';testcase;']) ? $group[';testcase;'] : false;
	unset($group[';testcase;']);

	$num_hosts = count($group['HOST_NAME']);

	# Set up structure ('host1' => array(1,5,8), 'host2' =>array(2,3,4,7), ...)
	# where the numbers are indices of services in original array.
	$host_idxs = array();
	for($i=0 ; $i<$num_hosts ; $i++)
	{
		$h = $group['HOST_NAME'][$i];
		if(array_key_exists($h, $host_idxs))
		{
			$host_idxs[$h][] = $i;
		}
		else
		{
			$host_idxs[$h] = array($i);
		}
	}

	$new_order = array(); # The new sorting order. used to re-order every array in $group
	ksort($host_idxs);

	if(!array_key_exists('SERVICE_DESCRIPTION', $group))
	{
		$new_order = array_values($host_idxs);
		for($i=0,$n=count($new_order) ; $i<$n ; $i++)
		{
			$new_order[$i] = $new_order[$i][0];
		}
	}
	else #services or servicegroups
	{
		# For every host: re-order service names by alphabet
		foreach($host_idxs as $h => $serv_indices)
		{
			$tmp_servs = array();
			foreach($serv_indices as $i)
			{
				$tmp_servs[$i] = $group['SERVICE_DESCRIPTION'][$i];
			}
			asort($tmp_servs);
			$new_order = array_merge($new_order, array_keys($tmp_servs));
		}
	}
	# $new_order now contains the indices to move elements of
	# arrays as for them to become correctly ordered.

	# use new order to reorder all arrays
	$a_names = array_keys($group);
	foreach($a_names as $a_name)
	{
		$arr =& $group[$a_name];
		if(!is_array($arr)) # only re-order arrays
			continue;

		$tmp_arr = array();
		foreach($new_order as $new_index => $old_index)
		{
			# print "moving ".$arr[$old_index]." from $old_index to $new_index\n";
			$tmp_arr[$new_index] = $arr[$old_index];
		}

		ksort($tmp_arr);
		$group[$a_name] = $tmp_arr;
	}

	$group[';testcase;'] = $testcase;
}

function get_csv_link($path=false, $params=false)
{
	$path = addslashes(trim($path));
	$params = addslashes(trim($params));
	if (empty($path) || empty($params))
		return false;
	$return = "<form action=\"".$path."\" method=\"post\" style=\"display:block; position: absolute; top: 0px; left: 720px\"><div>\n";

	$params_arr = explode('&', $params);
	foreach ($params_arr as $data) {
		$key_val = explode('=', $data);
		if (!empty($key_val)) {
			$key = $key_val[0];
			$val = $key_val[1];
			$return .= "<input type=\"hidden\" name=\"".$key."\" value=\"".$val."\" />\n";
		}
	}

	$return .= "<input type='image' src='/monitor/op5/reports/gui/icons/32x32/page-csv.png' alt='Download report as CSV' title='Download report as CSV' style='border: 0px; width: 32px; height: 32px; margin-top: -1px;' /></div></form>\n";
	return $return;
}
?>
