<?php

/**
 * Generate "show as pdf" link with icon, as a small html form.
 *
 * @param string $report   The type of report to produce. Currently supported values are 'sla' and 'avail'.
 * @param string $user_url The url to convert to PDF. If none is given, the calling script is used. All request variables are passed to the url.
 * @param array $user_options Custom options sent to html2ps
 * @param string $action_url The html2ps script that handles the link
 * @return string Complete HTML for the resulting link
 */
function get_pdf_link($report, $user_url=false, $user_options=false, $user_action_url=false)
{
	$pdf_img_src = '/monitor/op5/reports/gui/icons/32x32/page-pdf.png';
	$pdf_img_alt = 'Show as pdf';

	$default_filename = 'report.pdf';
	$default_options = array
	(
		'pixels' 					=> 1024, # can be any integer
		# 'method' 			=> 'fpdf', #default. doesn't use pdfversion
		#'media' 				=> 'A4', # default
		'renderimages' 	=> true,
#		'renderforms' 		=> true,
#		'renderfields' 		=> true,
#		'renderlinks'			=> true,
		'cssmedia'			=> 'projection', # to be able to use separate css
		'output' 				=> 0, # inline view
		'filename' 			=> $default_filename,
		# html2ps do not seem to obey the following line
		'margins'				=> array('leftmargin'=>20, 'rightmargin'=>20, 'topmargin'=>20, 'bottommargin'=>20),
		'smartpagebreaks' => true,
#		'debugbox'			=> 1,
	);
	#$default_action_url = 'http://192.168.1.29/html2ps/html2ps.php';
#	$default_action_url = '/op5common/lib/html2ps/html2ps.php';
	$default_action_url = '/monitor/op5/reports/gui/create_pdf.php';

	$url = $_SERVER['SERVER_ADDR'].$_SERVER['PHP_SELF'];

	if($user_url)
		$url = $user_url;


	$options = $default_options;
	if($user_options)
	{
		foreach($user_options as $opt => $val)
			$options[$opt] = $val;
	}

	$action_url = $default_action_url;
	if($user_action_url)
		$action_url = $user_action_url;

	# make sure action exists, keeps us from creating broken links on systems where op5common has not been updated
	if(!file_exists("/var/www/html$action_url"))
		return "";

	# start of deprecated code needed for old pdf backend:

	$form = "<form action='$action_url' method='post' style='display:block; position: absolute; top: 0px; left: 760px;'>\n";
	$form .= '<div>';
	$form .= "<input type='hidden' name='report' value='$report' />\n";
	$url_params = '';
	$url_params_to_skip = array('js_start_time', 'js_end_time', 's1'); # params that just f--k up things
	foreach($_REQUEST as $key => $val)
	{
		if(is_array($val))
		{
			# note: only support arrays of depth==1
			foreach($val as $subval)
			{
				$form .= "<input type='hidden' name='{$key}[]' value='$subval' />\n";
			}
		}
		else
		{
			if(in_array($key, array('start_time', 'end_time')))
				$val = strtotime($val);

			if(!in_array($key, $url_params_to_skip))
				$form .= "<input type='hidden' name='$key' value='$val' />\n";
		}
	}

	foreach($options as $opt => $val)
	{
		if(is_array($val))
		{
			foreach($val as $subkey => $subval)
				$form .= '<input type="hidden" name="'.$opt[$subkey].'" value="'.$subval.'" />'."\n";
		}
		else
			$form .= "<input type='hidden' name='$opt' value='$val' />\n";
	}

	$form .= "<input type='image' src='$pdf_img_src' alt='$pdf_img_alt' title='$pdf_img_alt' style='border: 0px; width: 32px; height: 32px; margin-top: -1px;' />";
	$form .= '</div>';
	$form .= "</form>";

	return $form;
}
?>
