<?php

require_once 'timeperiod_class.php';
require_once('paths.inc.php');
require_once (dirname( __FILE__) . '/sql_class.php');
require_once($op5common_path . '/lib/common.php');

$sql_class = new sql_class();
$dba = $sql_class->db;
if (!$dba) {
	// db connection failed - bail out
	echo "Database connection failed";
	exit;
}

// state codes
define('STATE_PENDING', -1);
define('STATE_OK', 0);
define('HOST_UP', 0);
define('HOST_DOWN', 1);
define('HOST_UNREACHABLE', 2);
define('HOST_PENDING', -1);
define('SERVICE_OK', 0);
define('SERVICE_WARNING', 1);
define('SERVICE_CRITICAL', 2);
define('SERVICE_UNKNOWN', 3);
define('SERVICE_PENDING', -1);
define('PROCESS_SHUTDOWN',	103);
define('PROCESS_RESTART',   102);
define('PROCESS_START',		100);
define('SERVICECHECK',		701);
define('HOSTCHECK', 		801);
define('DOWNTIME_START',	1103);
define('DOWNTIME_STOP',		1104);

define('GUI_DB',	 		'monitor_gui');

define('PERC_DEC',			3); // nr of decimals in returned percentage

define('DEBUG', 		false);

function lib_reports_assert_handler($filename, $line, $code)
{
	if (!posix_isatty(0))
		echo "<pre>\n";

	echo "ASSERT FAILED\n";
	debug_print_backtrace();

	echo "File: $filename\n\n";
	echo "Line: $line\n";
	echo "Assertion: $code\n";

	if (!posix_isatty(0))
		echo "</pre>\n";
}

if (DEBUG === true) {
	assert_options(ASSERT_ACTIVE, 1);
	assert_options(ASSERT_WARNING, 0);
	assert_options(ASSERT_QUIET_EVAL, 0);
	assert_options(ASSERT_BAIL, 1);
	assert_options(ASSERT_CALLBACK, 'lib_reports_assert_handler');
}

class report_class
{
	var $db_start_time = 0; # earliest database timestamp we look at
	var $db_end_time = 0;   # latest database timestamp we look at
	var $debug = array();

	var $st_raw = array(); # raw states
	var $st_needs_log = false;
	var $st_needs_log_maxrows = 0;
	var $st_needs_log_states_mask = 0xffffffff;
	var $st_log = false;
	var $st_prev_row = array();
	var $st_running = 0;
	var $st_dt_depth = 0;
	var $st_is_service = false;
	var $st_source = false;
	var $st_inactive = 0;
	var $st_text = array();
	var $st_sub = array(); # only used by the master report
	var $st_sub_discrepancies = 0;
	var $st_obj_type = '';
	var $st_last_dt_start_depth = array();

	/**
	 * The calculated state of the object, taking such things
	 * as scheduled downtime counted as uptime into consideration
	 */
	var $st_obj_state = STATE_PENDING;

	/** The real state of the object */
	var $st_real_state = STATE_PENDING;

	var $state_tpl_host = array(
								'HOST_NAME' => '',
								'TIME_UP_SCHEDULED' => 0,
								'TIME_UP_UNSCHEDULED' => 0,
								'TIME_DOWN_SCHEDULED' => 0,
								'TIME_DOWN_UNSCHEDULED' => 0,
								'TIME_UNREACHABLE_SCHEDULED' => 0,
								'TIME_UNREACHABLE_UNSCHEDULED' => 0,
								'TIME_UNDETERMINED_NOT_RUNNING' => 0,
								'TIME_UNDETERMINED_NO_DATA' => 0,
								);

	var $state_tpl_svc = array(
							   'HOST_NAME' => '',
							   'SERVICE_DESCRIPTION' => '',
							   'TIME_OK_SCHEDULED' => 0,
							   'TIME_OK_UNSCHEDULED' => 0,
							   'TIME_WARNING_SCHEDULED' => 0,
							   'TIME_WARNING_UNSCHEDULED' => 0,
							   'TIME_UNKNOWN_SCHEDULED' => 0,
							   'TIME_UNKNOWN_UNSCHEDULED' => 0,
							   'TIME_CRITICAL_SCHEDULED' => 0,
							   'TIME_CRITICAL_UNSCHEDULED' => 0,
							   'TIME_UNDETERMINED_NOT_RUNNING' => 0,
							   'TIME_UNDETERMINED_NO_DATA' => 0,
							   );

	var $master = false;
	var $id = '';
	var $result = array();
	var $csv_result = array();
	var $old_csv_result = array();
	var $sub_results = array();
	var $use_average = false;
	var $assume_states_during_not_running = false;
	var $initial_assumed_host_state = false;
	var $initial_assumed_service_state = false;
	var $scheduled_downtime_as_uptime = false;
	var $include_soft_states = true; /* NOTE: defaults to true */
	var $report_timeperiod = false;
	var $timeperiods_resolved = false; #whether timeperiod exceptions and exclusions have been resolved
	var $initial_state = false;
	var $initial_dt_depth = false;
	var $start_time = false;
	var $end_time = false;
	var $host_name = false;
	var $service_description = false;
	var $servicegroup = false;
	var $hostgroup = false;
	var $db_name = 'merlin';
	var $db_table = 'report_data';
	var $sub_reports = array();
	var $last_shutdown = false;
	var $states = array();
	var $tp_exceptions = array();

	var $options = array();

	/**
	*	@name	report_class
	*	@desc	constructor
	*
	*/
	function report_class($db_name='merlin', $db_table='report_data')
	{
		if (!empty($db_name))
			$this->db_name 	= $db_name;
		if (!empty($db_table))
			$this->db_table = $db_table;
	}

	/**
	 * Parses a timerange string
	 * FIXME: add more validation
	 * @param string $str
	 * @return array An array of timeranges
	 * E.g:
	 * $str="08:00-12:00,13:00-17:00" gives:
	 * array
	 * (
	 * 		array('start' => '08:00', 'stop' => '12:00'),
	 * 		array('start' => '13:00', 'stop' => '17:00')
	 * );
	 */
	function tp_parse_day($str)
	{
		if (!$str)
			return 0;

		$i = 0;
		$ret = array();

		$ents = explode(',', $str);
		foreach ($ents as $ent) {
			$start_stop = explode('-', $ent);
			$start_hour_minute = explode(':', $start_stop[0]);
			$start_hour = $start_hour_minute[0];
			$start_minute = $start_hour_minute[1];
			$stop_hour_minute = explode(':', $start_stop[1]);
			$stop_hour = $stop_hour_minute[0];
			$stop_minute = $stop_hour_minute[1];
			$stop_hour_minute = $start_hour_minute = $stop = $start = false;
			$start_second = ($start_hour * 3600) + ($start_minute * 60);
			$stop_second = ($stop_hour * 3600) + ($stop_minute * 60);
			if($start_second >= $stop_second)
			{
				print "Error: Skipping timerange $str, stop time is before start time<br>";
				continue;
			}
			$ret[$i]['start'] = $start_second;
			$ret[$i]['stop'] = $stop_second;
			$i++;
		}

		return $ret;
	}

	function set_report_timeperiod($period)
	{
		global $dba, $sql_class, $valid_weekdays;

		if (empty($period)) {
			$this->report_timeperiod = false;
			return true;
		}

		$this->report_timeperiod = array();

		$query = 'SELECT * FROM timeperiod ' .
			'WHERE timeperiod_name = \'' .
			$sql_class->eScaPeSTr($period) . "'"; /* OH EM GEE!! CamuLCASE! */

		$result = $dba->execute($query);
		if (!$result) {
			echo "No result\n";
			return false;
		}
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			return false;
		}

		$result = array_pop($result);
		$timeperiod_id = $result['id'];
		unset($result['id']);
		unset($result['timeperiod_name']);
		unset($result['alias']);

		$includes = $result;

		$query = "SELECT tp.* FROM timeperiod tp".
				 " JOIN timeperiod_exclude ON exclude = id ".
				 " WHERE timeperiod = $timeperiod_id";

		$result_set = $dba->execute($query);

		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			return false;
		}

		if($result_set)
		{
			foreach($result_set as $result_row) # for each exclude
			{
				foreach($valid_weekdays as $i => $weekday) # each weekday
				{
					if(!isset($includes[$weekday]))
					{
						# no time to include this day
						continue;
					}
					$exclude_ranges =& $result_row[$weekday];
					if (empty($exclude_ranges))
					{
						# have include, no exclude
						$this->report_timeperiod[$i] = $this->tp_parse_day($includes[$weekday]);
					}
					else
					{
						# have both include and exclude
						$include        = $this->tp_parse_day($includes[$weekday]);
						$exclude_ranges = $this->tp_parse_day($exclude_ranges);
						$this->report_timeperiod[$i] = subtract_timerange_sets($include, $exclude_ranges);
					}
				}
			}
		}
		else # use old set_option() based functionality
		{
			$errors = 0;
			foreach ($includes as $k => $v) {
				if (empty($v)) {
					continue;
				}
				$errors += $this->set_option($k, $v) === false;
			}

			if ($errors)
				return false;
		}

		return true;
	}

	/**
	 * Finds next start or stop of timeperiod from a given timestamp. If given time is
	 * in an inactive timeperiod and we're looking for a stop, current time is returned.
	 * Vice versa, if we're looking for start inside an active period, the current timestamp is returned.
	 *
	 * @param int $when Current timestamp to start from.
	 * @param string$what Whether to search for start or stop. Valid values are 'start' and 'stop'.
	 * @return int The timestamp
	 */
	function tp_next($when, $what = 'start')
	{
		if ($this->report_timeperiod === false)
			return $when;

		# if there is a report timeperiod set that doesn't have
		# any 'start' entry (ie, all days are empty, such as for
		# the "none" timeperiod), we can't possibly find either
		# start or stop, so we can break out early.
		# Noone sane will want to take a report from such a timeperiod,
		# but in case the user misclicks, we should behave properly.
		if (empty($this->report_timeperiod))
			return 0;

		$other = 'stop';
		if ($what === 'stop')
			$other = 'start';

		$tm = localtime($when, true);
		$year = $tm['tm_year'] + 1900; # stored as offsets since 1900
		$tm_yday = $tm['tm_yday'];
		$day = $tm['tm_wday'];
		$day_seconds = ($tm['tm_hour'] * 3600) + ($tm['tm_min'] * 60) + $tm['tm_sec'];

		$midnight_to_when = $when - $day_seconds;
		$ents = array();
		# see if we have an exception first
		if (!empty($this->tp_exceptions[$year][$tm_yday])) {
			$ents = $this->tp_exceptions[$year][$tm_yday];
		}
		elseif (!empty($this->report_timeperiod[$day]))
			$ents = $this->report_timeperiod[$day];

		foreach ($ents as $ent) {
			if ($ent[$what] <= $day_seconds && $ent[$other] > $day_seconds)
				return $when;

			if ($ent[$what] > $day_seconds)
				return $midnight_to_when + $ent[$what];
		}

		$orig_day = $day;
		$loops = 0;
		for ($day = $orig_day + 1; $loops < 7; $day++) {
			$ents = false;
			$loops++;
			if ($day > 6)
				$day = 0;

			$midnight_to_when += 86400;

			if (!empty($this->tp_exceptions[$year][$tm_yday + $day]))
				$ents = $this->tp_exceptions[$year][$tm_yday + $day];
			elseif (!empty($this->report_timeperiod[$day]))
				$ents = $this->report_timeperiod[$day];

			# no exceptions, no timeperiod entry
			if (!$ents)
				continue;

			foreach ($ents as $ent)
				return $midnight_to_when + $ent[$what];
		}

		echo "upsadaisy, I fell through\n";

		return 0;
	}

	/**
	 * @name   tp_active_time
	 * @desc   Returns the number of active seconds "inside"
	 *         the timeperiod during the start -> stop interval
	 * @param  int $start: A timestamp in the unix epoch notation
	 * @param  int $stop: A timestamp in the unix epoch notation
	 * @return The number of seconds included in both $stop - $start
	 *         and the timeperiod set for this report as an integer
	 *         in the unix epoch notation
	 */
	function tp_active_time($start, $stop)
	{
		# if no timeperiod is set, the entire duration is active
		if ($this->report_timeperiod === false)
			return $stop - $start;

		# a timeperiod without entries will cause us to never
		# find start or stop. otoh, it never has any active time,
		# so we simply return 0
		if ($start >= $stop || $this->report_timeperiod === false)
			return 0;

		$nstart = $this->tp_next($start, 'start');
		# if there is no start event inside the timeperiod, or the
		# first ever $nstart is beyond our $stop parameter,
		# there are no active seconds between start and stop, so
		# break out early
		if (!$nstart || $nstart >= $stop)
			return 0;

		$nstop = $this->tp_next($nstart, 'stop');
		# If the first ever $nstop encountered is beyond our
		# $stop parameter, we can return early, as we won't
		# need to loop at all
		if ($nstop > $stop)
			return $stop - $nstart;

		$active = $nstop - $nstart;
		for (;;) {
			if (($nstart = $this->tp_next($nstop, 'start')) > $stop)
				$nstart = $stop;
			if (($nstop = $this->tp_next($nstart, 'stop')) > $stop)
				$nstop = $stop;

			$active += $nstop - $nstart;
			if ($nstart >= $stop || $nstop >= $stop)
				return $active;
		}

		/* never reached */

		echo "HALALIIIII!\n";
		exit(1);
	}

	function register_db_time($t)
	{
		if (!$this->db_start_time || $t < $this->db_start_time)
			$this->db_start_time = $t;
		if (!$this->db_end_time || $t > $this->db_end_time)
			$this->db_end_time = $t;
		$this->debug['db_start_time'] = $this->db_start_time;
		$this->debug['db_end_time'] = $this->db_end_time;
	}

	function set_option($name, $value)
	{
		$vtypes = array('keep_logs' => 'bool',
						'keep_logs_maxrows' => 'int',
						'keep_logs_states' => 'array',
						'report_timeperiod' => 'string',
						'scheduled_downtime_as_uptime' => 'bool',
						'assume_initial_states' => 'bool',
						'assume_states_during_not_running' => 'bool',
						'initial_assumed_host_state' => 'string',
						'initial_assumed_service_state' => 'string',
						'include_soft_states' => 'bool',
						'host_name' => 'list',
						'service_description' => 'list',
						'hostgroup_name' => 'string',
						'servicegroup_name' => 'string',
						'start_time' => 'int',
						'end_time' => 'int',
						'monday' => 'string',
						'tuesday' => 'string',
						'wednesday' => 'string',
						'thursday' => 'string',
						'friday' => 'string',
						'saturday' => 'string',
						'sunday' => 'string',
						'exclude' => 'string',
						'use_average' => 'bool');

		# trigger_error(sprintf("trying to set variable %s to %s", $name, $value));
		# this will happen for timeperiod exceptions
		if (!isset($vtypes[$name]))
			return $this->set_timeperiod_variable($name, $value);

		switch ($vtypes[$name]) {
		 case 'bool':
			if ($value == 1 || !strcasecmp($value, "true") || !empty($value))
				$value = true;
			else
				$value = false;
			if (!is_bool($value))
				return false;
			break;
		 case 'int':
			if (!is_numeric($value) || $value != intval($value))
				return false;
			$value = intval($value);
			break;
		 case 'string':
			if (!is_string($value))
				return false;
			break;
		 case 'list':
			if (is_array($value) && count($value) === 1)
				$value = array_pop($value);
			if (is_string($value))
				break;
			/* fallthrough */
		 case 'array':
			if (!is_array($value))
				return false;
			break;
		 default:
			# this is an exception and should never ever happen
			return false;
		}

		switch ($name) {
		 case 'keep_logs':
			# caller forces us to retain or discard all log-entries
			$this->st_needs_log = $value;
			break;
		 case 'keep_logs_maxrows':
			$this->st_needs_log_maxrows = (int)$value;
			break;
		 case 'keep_logs_states':
			$tmp_mask = 0;

			foreach ($value as $mask) {
				$tmp_mask |= (int) $mask;
			}

			$this->st_needs_log_states_mask = $tmp_mask;
			# echo "setting option to keep_logs_states_mask " . $tmp_mask . "<br>";
			break;
		 case 'scheduled_downtime_as_uptime':
			$this->scheduled_downtime_as_uptime = $value;
			break;
		 case 'assume_initial_states':
			$this->assume_initial_states = $value;
			if (!$this->assume_initial_states) {
				$this->initial_assumed_host_state = false;
				$this->initial_assumed_service_state = false;
			}
			break;
		 case 'initial_assumed_host_state': case 'initial_assumed_service_state':
			if ($value < -3 || !$this->assume_initial_states)
				return false;
			$this->$name = $value;
			break;
		 case 'report_timeperiod':
			return $this->set_report_timeperiod($value);
			break;
		 case 'start_time':
			$this->start_time = $value;
			break;
		 case 'end_time':
			$this->end_time = $value;
			break;
		 case 'use_average':
			$this->use_average = $value;
			break;
		 case 'assume_states_during_not_running':
			$this->assume_states_during_not_running = $value;
			break;
		 case 'include_soft_states':
			$this->include_soft_states = $value;
			break;
		 case 'host_name':
			$this->hostgroup_name = false;
			$this->servicegroup_name = false;
			$this->host_name = $value;
			return true;
			break;
		 case 'service_description':
			$this->hostgroup_name = false;
			$this->servicegroup_name = false;
			$this->service_description = $value;
			if (!is_array($value) && !$this->host_name && strchr($value, ';')) {
				$parts = explode(';', $value);
				$this->host_name = $parts[0];
				$this->service_description = $parts[1];
			}
			return true;
			break;
		 case 'hostgroup_name':
			$this->host_name = false;
			$this->service_description = false;
			$this->servicegroup_name = false;
			$this->hostgroup_name = $value;
			return true;
			break;
		 case 'servicegroup_name':
			$this->host_name = false;
			$this->service_description = false;
			$this->hostgroup_name = false;
			$this->servicegroup_name = $value;
			return true;
			break;
		 case 'sunday':
			$this->report_timeperiod[0] = $this->tp_parse_day($value);
			break;
		 case 'monday':
			$this->report_timeperiod[1] = $this->tp_parse_day($value);
			break;
		 case 'tuesday':
			$this->report_timeperiod[2] = $this->tp_parse_day($value);
			break;
		 case 'wednesday':
			$this->report_timeperiod[3] = $this->tp_parse_day($value);
			break;
		 case 'thursday':
			$this->report_timeperiod[4] = $this->tp_parse_day($value);
			break;
		 case 'friday':
			$this->report_timeperiod[5] = $this->tp_parse_day($value);
			break;
		 case 'saturday':
			$this->report_timeperiod[6] = $this->tp_parse_day($value);
			break;
		# FIXME: support exclude
		# FIXME: support exceptions
		 default:
			return false;
		}

		$this->options[$name] = $value;

		return true;
	}

	/**
	 * Adds a timeperiod exception to the report.
	 * FIXME: should probably validate more
	 * @param int $dateperiod_type Indicates the type of exception. Se timeperiod_class.php for valid values.
	 * @param int $syear Start year
	 * @param int $smon Start month
	 * @param int $smday Start day of month
	 * @param int $swday Start weekday
	 * @param int $swday_offset Start weekday offset
	 * @param int $eyear End year
	 * @param int $emon End month
	 * @param int $emday End day of month
	 * @param int $ewday End weekday
	 * @param int $ewday_offset End weekday offset
	 * @param int $skip_interval Interval to skip, such as: "every 3 weeks" etc
	 * @param string $timeranges
	 * Throws Exception if any parameter has bogus values.
	 */
	function add_timeperiod_exception($dateperiod_type,
	                                  $syear, $smon, $smday, $swday, $swday_offset,
	                                  $eyear, $emon, $emday, $ewday, $ewday_offset,
	                                  $skip_interval, $timeranges)
	{
		global $days_per_month;

		if (!isset($this->tp_exceptions['unresolved']))
			$this->tp_exception['unresolved'] = array();

		assert($dateperiod_type >= 0 && $dateperiod_type < Timeperiod::DATERANGE_TYPES); # can only fail if programmer messed up
		$timeranges = $this->tp_parse_day($timeranges);

		$this->tp_exceptions['unresolved'][] = array
		(
			'type' => $dateperiod_type,
			'syear' => $syear,
			'smon' => $smon,
			'smday' => $smday,
			'swday' => $swday,
			'swday_offset' => $swday_offset,
			'eyear' => $eyear,
			'emon' => $emon,
			'emday' => $emday,
			'ewday' => $ewday,
			'ewday_offset' => $ewday_offset,
			'skip_interval' => $skip_interval,
			'timeranges' => $timeranges,
		);
		$this->timeperiods_resolved = false;
	}

	function set_options($options)
	{
		$errors = false;
		foreach ($options as $name => $value)
			$errors |= intval(!$this->set_option($name, $value));

		return $errors ? false : true;
	}

	function resolve_timeperiods()
	{
		if($this->start_time == false || $this->end_time == false)
		{
			throw new Exception("Timeperiods cannot be resolved unless report start and end time is set");
		}
		$start_time = $this->start_time;
		$end_time = $this->end_time;

		if($end_time < $start_time)
		{
			throw new Exception("Report time set to end before start");
		}

		$this->timeperiods_resolved = empty($this->tp_exceptions['unresolved']);
		if ($this->timeperiods_resolved)
			return;

		$all_exceptions =& $this->tp_exceptions;
		$unres_exceptions =& $all_exceptions['unresolved'];

		$start_year = date('Y', $start_time);
		$end_year = date('Y', $end_time);
		/*
		 * Goal:
		 * For every day of year affected by this exception,
		 * add timeranges to that day (if overlap with existing range, merge them)
		 *
		 * Plan: for every day within report period
		 * {
		 *		check conditions for current day
		 * 		if day should be included add timeranges
		 * }
		 */
		for($i=0,$n=count($unres_exceptions) ; $i<$n ; $i++)
		{
			$x =& $unres_exceptions[$i];

			if($x['syear'] > date('Y', $end_time))
				continue;

			for($day_time = $start_time ; $day_time < $end_time ; $day_time += 86400)
			{
				$check_exception = true;

				$day = date('z', $day_time);
				$day_year  = date('Y', $day_time);
				$day_month = date('n', $day_time);

				# find out if there is an exception during this day
				switch($x['type'])
				{
					# FIXME: More consequent scheme for adding 24h to end time

				 case Timeperiod::DATERANGE_CALENDAR_DATE:/* eg: 2008-12-25 */
					# set fields: syear, smon, smday, eyear, emon, emday, skip_interval

					$exp_start = mktime(0,0,0, $x['smon'], $x['smday'], $x['syear']);

					# unspecified end date - two possibilities
					if(is_daterange_single_day($x))
					{
						if($x['skip_interval'] > 1)
						{
							// yyyy-mm-dd / dd means endless period
							$exp_end = $day_time + 86400;
						}
						else
						{
							// Whereas yyyy-mm-dd means single day period
							$exp_end = $exp_start;
						}
					}
					else
						$exp_end = mktime(24,0,0, $x['emon'], $x['emday'], $x['eyear']);

					break;
				 case Timeperiod::DATERANGE_MONTH_DATE:
					/* eg: july 4 (specific month) */
					# set fields: smon, emon, smday, emday

					$exp_start = calculate_time_from_day_of_month($start_year, $x['smon'], $x['smday']);
					$exp_end   = calculate_time_from_day_of_month($end_year, $x['emon'], $x['emday']);

					# XXX: can *both* be zero here?
					if($exp_end < $exp_start)
					{
						$x['eyear'] = $day_year + 1;
						$exp_end = calculate_time_from_day_of_month($x['eyear'], $x['emon'], $x['emday']);
						$exp_end += 86400;
					}

					if($exp_end == 0) {
						echo "php is broken. goodie....\n";
						if($x['emday'] < 0) {
							$check_exception = false;
						}
						else {
							$exp_end = calculate_time_from_day_of_month($x['eyear'], $x['emon'], -1);
						}
					}

					assert($exp_end != 0);

					break;
				 case Timeperiod::DATERANGE_MONTH_DAY:
					/* eg: day 21 (generic month) */
					# set field: smday, emday
					$exp_start = calculate_time_from_day_of_month($day_year, $day_month, $x['smday']);
					$exp_end   = calculate_time_from_day_of_month($day_year, $day_month, $x['emday']);

					# get midnight at end of day
					$exp_end += 86400;
					break;

				 case Timeperiod::DATERANGE_MONTH_WEEK_DAY:/* eg: 3rd thursday (specific month) */
					# set field: smon, swday, swday_offset, emon, ewday, ewday_offset, skip_interval
					$exp_start = calculate_time_from_weekday_of_month($start_year, $x['smon'], $x['swday'], $x['swday_offset']);
					$exp_end   = calculate_time_from_weekday_of_month($end_year,   $x['emon'], $x['ewday'], $x['ewday_offset']);
					break;

				 case Timeperiod::DATERANGE_WEEK_DAY:
					# eg: 3rd thursday (generic month)
					# set fields: swday, swday_offset, ewday, ewday_offset, skip_interval
					$exp_start = calculate_time_from_weekday_of_month($day_year, $day_month, $x['swday'], $x['swday_offset']);
					$exp_end   = calculate_time_from_weekday_of_month($day_year, $day_month, $x['ewday'], $x['ewday_offset']);
					break;
				}

				# This day might be totally uninteresting, in which case
				# we just ignore it
				if($x['skip_interval'] > 1) {
					$days_since_start = ($day_time - $exp_start) / 86400;
					$check_exception = !($days_since_start % $x['skip_interval']);
				}

				if (!$check_exception || $exp_start > $day_time || $exp_end < $day_time)
					continue;

				if(!isset($all_exceptions[$day_year]))
					$all_exceptions[$day_year] = array();

				if(!isset($all_exceptions[$day_year][$day]))
					$all_exceptions[$day_year][$day] = array();

				# if so, merge timeranges with existing for this day
				$all_exceptions[$day_year][$day] = merge_timerange_sets($all_exceptions[$day_year][$day], $x['timeranges']);
			}
		}
		unset($this->tp_exceptions['unresolved']);
		$this->timeperiods_resolved = true;
	}


	function set_master($master)
	{
		$this->master = $master;
		$this->set_options($master->options);
		$this->last_shutdown = $master->last_shutdown;
		$this->report_timeperiod = $master->report_timeperiod;
	}

	/**
	 *	@name 	get_uptime
	 *	@desc 	Calculate uptime between two timestamps for host/service
	 * 	@param 	str $hostname If set to false, internal host_name is used.
	 * 	@param	str $servicename If set to false, internal service_description is used
	 * 	@param 	mixed $start_time datetime or unix timestamp
	 * 	@param 	mixed $end_time datetime or unix timestamp
	 *	@param mixed $hostgroup If set to false, internal hostgroup is used
	 *	@param mixed $servicegroup If set to false, internal servicegroup is used
	 * 	@return array or false on error
	 *
	 */
	function get_uptime($hostname=false, $servicename=false, $start_time=0,
						$end_time=0, $hostgroup=false, $servicegroup=false)
	{
		if (empty($hostname) && !empty($this->host_name))
			$hostname = $this->host_name;
		if (empty($servicename) && !empty($this->service_description))
			$servicename = $this->service_description;
		if (empty($hostgroup) && !empty($this->hostgroup))
			$hostgroup = $this->hostgroup;
		if (empty($servicegroup) && !empty($this->servicegroup))
			$servicegroup = $this->servicegroup;
		if (empty($start_time) && !empty($this->start_time))
			$start_time = $this->start_time;
		if (empty($end_time) && !empty($this->end_time))
			$end_time = $this->end_time;

		if (empty($hostname) && empty($hostgroup) && empty($servicename) && empty($servicegroup)) {
			return false;
		}

		# stash the report settings for debugging/test-creation purposes
		$this->options['start_time'] = $this->start_time = $start_time;
		$this->options['end_time'] = $this->end_time = $end_time;
		$this->debug = $this->options;
		if (!is_array($hostname))
			$this->debug['host_name'] = $this->host_name = $hostname;
		if (!is_array($servicename))
			$this->debug['service_description'] = $this->service_description = $servicename;
		$this->debug['servicegroup_name'] = $this->servicegroup = $servicegroup;
		$this->debug['hostgroup_name'] = $this->hostgroup = $hostgroup;

		# We need to be able to see what host/service group we are fetching
		# data for. This is actually a hack since it should be set by using
		# set_option() but this will have to do for now...
		if (!empty($hostgroup)) $this->hostgroup = $hostgroup;
		if (!empty($servicegroup)) $this->servicegroup = $servicegroup;

		// check and possibly convert datetime for start/end_time
		$start_time = $this->datetime_to_timestamp($start_time);
		$end_time 	= $this->datetime_to_timestamp($end_time);

		# these get copied to sub objects automagically
		# using the set_options() method
		$this->set_option('start_time', $start_time);
		$this->set_option('end_time', $end_time);

		$this->get_last_shutdown();

		$host 		= false;
		$service	= false;
		$res_group	= false;

		# When we have a single host or service, we don't need to
		# calculate group availability, so do that up front to
		# get the simple case out of the way immediately
		if (empty($hostgroup) && empty($servicegroup) &&
		    !is_array($hostname) && !is_array($servicename))
		{
			// == single host OR service ==
			$ret = $this->calculate_uptime($hostname, $servicename);
			foreach ($this->debug as $k => $v) {
				if ($v === false)
					unset($this->debug[$k]);
			}

			$ret[';testcase;'] = $this->debug;
			return $ret;
		}

		if (!empty($hostgroup)) {
			$hostname = array();
			// == Hostgroup ==
			$res_group = get_hostgroup_members($hostgroup);
			if (!$res_group)
				return false;

			foreach ($res_group as $row) {
				$hostname[] = $row['host_name'];
			}
		} elseif (!empty($servicegroup)) {
			$servicename = array();
			// == Servicegroup ==
			$res_group = get_servicegroup_members($servicegroup);
			if (!$res_group)
				return false;

			foreach ($res_group as $row) {
				$servicename[] = $row['host_name'] . ';' . $row['service_description'];
			}
		}

		if (is_array($hostname) && is_array($servicename))
			return false;

		if (is_array($hostname)) {
			// == multiple hosts ==
			foreach ($hostname as $host) {
				$sub_class = new report_class($this->db_name, $this->db_table);
				$sub_class->set_master($this);
				$sub_class->id = $host;
				$sub_class->st_init($host, false);
				$this->sub_reports[$sub_class->id] = $sub_class;
			}
		} else {
			if (is_array($servicename)) {
				// == multiple services ==
				foreach ($servicename as $service) {
					// split hostname, service_desciption on ';'
					$service_parts = explode(';', $service);
					$sub_class = new report_class($this->db_name, $this->db_table);
					$sub_class->set_master($this);
					$sub_class->id = $service;
					$sub_class->st_init($service_parts[0], $service_parts[1]);
					$this->sub_reports[$sub_class->id] = $sub_class;
				}
			}
		}

		# Grab master's report-results _FIRST_ as sub-reports
		# are fed its data from the same query
		$this->st_init($hostname, $servicename);
		$this->st_parse_all_rows($hostname, $servicename);
		$sub_return = false;
		foreach ($this->sub_reports as $id => $rpt) {
			$return[] = $rpt->st_finalize();
			$this->register_db_time($rpt->db_start_time);
			$this->register_db_time($rpt->db_end_time);
			if (strpos($id, ';') !== false) {
				$this->debug['service_description'][] = $id;
			} else {
				$this->debug['host_name'][] = $id;
			}
		}
		$master_return = $this->st_finalize();
		foreach ($master_return as $k => $v)
			$return[$k] = $v;

		# stash the debugging stuff in the return array, but only
		# for the master report
		if (empty($this->master)) {
			$this->debug['db_end_time'] = $this->db_end_time;
			$this->debug['db_start_time'] = $this->db_start_time;
			foreach ($this->debug as $k => $v) {
				if ($v === false)
					unset($this->debug[$k]);
			}
			$return[';testcase;'] = $this->debug;

			return $return;
		}
	}

	/**
	 *	@name get_last_shutdown
	 *	@desc returns last shutdown event prior to $start_time
	 *
	 */
	function get_last_shutdown()
	{
		global $dba;

		if ($this->last_shutdown != false)
			return $this->last_shutdown;

		# If we're assuming states during program downtime,
		# we don't really need to know when the last shutdown
		# took place, as the initial state will be used regardless
		# of whether or not Monitor was up and running.
		if ($this->assume_states_during_not_running) {
			$this->last_shutdown = 0;
			return 0;
		}

		$query = "SELECT timestamp, event_type FROM ".
			$this->db_name.".".$this->db_table.
			" WHERE timestamp <".$this->start_time.
			" ORDER BY timestamp DESC LIMIT 1";
		if (!$res = $dba->execute($query)) {
			if (!empty($dba->error))
				__debug(__FUNCTION__, $dba->error, $query);
			return false;
		}

		$this->register_db_time($res[0]['timestamp']);

		$event_type = $res[0]['event_type'];
		if ($event_type==PROCESS_SHUTDOWN || $event_type==PROCESS_RESTART)
			$this->last_shutdown = $res[0]['timestamp'];
		else
			$this->last_shutdown = 0;

		return $this->last_shutdown;
	}

	/**
	 * Calculate the time spent in different states as total and percentage.
	 *
	 * @param array $state State times. Has the format:<br>
	 * array("X:Y:Z" => seconds, 	...). Where X, Y and Z are numeric states and rhs argument is the number of seconds in that state
	 * @param array $conv State translation table. E.g. for hostgroups:<br> array(0 => 'UP', '1' => 'DOWN', '2' => 'UNREACHABLE', '-1' => 'PENDING')
	 * @return array A huge array with all possible states and time spent in that state. States called PERCENT_* contains percentages rather than a number of seconds.
	 */
	function convert_state_table($state, $conv)
	{
		$cstate = array();
		$cstate['TIME_UNDETERMINED_NO_DATA'] = 0;
		$cstate['TIME_UNDETERMINED_NOT_RUNNING'] = 0;
		$cstate['TOTAL_TIME_UNSCHEDULED'] = 0;
		$cstate['TOTAL_TIME_SCHEDULED'] = 0;
		$cstate['TOTAL_TIME_UNDETERMINED'] = 0;
		$cstate['TOTAL_TIME_KNOWN'] = 0;
		$cstate['TOTAL_TIME_ACTIVE'] = 0;
		foreach ($state as $s => $duration) {
			$known = false;
			$cstate['TOTAL_TIME_ACTIVE'] += $duration;
			$ary = explode(':', $s);
			$is_running = intval($ary[0]);
			$in_dt = $ary[1] == 1;
			$p3 = $in_dt ? '' : 'UN';
			$p3 .= 'SCHEDULED';

			if (!$is_running)
				$cstate['TIME_UNDETERMINED_NOT_RUNNING'] += $duration;

			$p1 = $is_running ? '_' : '_UNKNOWN_';

			# this is where we hack in scheduled downtime as uptime
			if ($in_dt && $this->scheduled_downtime_as_uptime) {
				$p2 = $conv[0];
			}
			elseif (isset($conv[$ary[2]])) {
				$p2 = $conv[$ary[2]];

				if ($p2 === 'PENDING')
					$cstate['TIME_UNDETERMINED_NO_DATA'] += $duration;
			}
			else {
				$p2 = "BAD_BUG_ERROR";
			}

			if (!$is_running || $p2 === 'PENDING') {
				$known = false;
				$cstate['TOTAL_TIME_UNDETERMINED'] += $duration;
			}
			else {
				$cstate['TOTAL_TIME_KNOWN'] += $duration;
				$known = true;
			}

			$tot_state = "TOTAL_TIME_$p2";
			if (!isset($cstate[$tot_state]))
				$cstate[$tot_state] = $duration;
			else
				$cstate[$tot_state] += $duration;

			if ($known) {
				$kstate = "KNOWN_TIME_$p2";
				if (!isset($cstate[$kstate]))
					$cstate[$kstate] = $duration;
				else
					$cstate[$kstate] += $duration;
			}

			# scheduled/unscheduled totals
			$cstate['TOTAL_TIME_' . $p3] += $duration;

			$cname = 'TIME' . $p1 . $p2 . '_' . $p3;
			if (!isset($cstate[$cname]))
				$cstate[$cname] = $duration;
			else
				$cstate[$cname] += $duration;

			if ($known) {
				$cname = 'KNOWN_' . $cname;
				if (!isset($cstate[$cname]))
					$cstate[$cname] = $duration;
				else
					$cstate[$cname] += $duration;
			}
		}

		$sched_junk = array('_SCHEDULED', '_UNSCHEDULED');
		foreach (array('KNOWN_', '') as $known) {
			foreach ($conv as $s) {
				foreach ($sched_junk as $dt_str) {
					$entry = $known . "TIME_$s" . $dt_str;
					if (!isset($cstate[$entry]))
						$cstate[$entry] = 0;
					$entry = "KNOWN_TIME_$s" . $dt_str;
				}
			}
		}

		# For each $state, we need to calculate
		# PERCENT_TOTAL_TIME_$state,
		# PERCENT_TIME_$state_SCHEDULED,
		# PERCENT_TIME_$state_UNSCHEDULED,
		# PERCENT_KNOWN_TIME_$state,
		# PERCENT_KNOWN_TIME_$state_SCHEDULED,
		# PERCENT_KNOWN_TIME_$state_UNSCHEDULED
		$conv['UNDETERMINED'] = 'UNDETERMINED';
		$div = $cstate['TOTAL_TIME_ACTIVE'];
		foreach ($conv as $state) {
			$str = 'TIME_' . $state;
			foreach (array('TOTAL_', 'KNOWN_') as $prefix) {
				$full_str = $prefix . $str;

				if (!isset($cstate[$full_str]))
					$cstate[$full_str] = 0;
				$cstate['PERCENT_' . $full_str] =
					percent($cstate[$full_str], $div);
			}

			foreach (array('', 'KNOWN_') as $known) {
				foreach ($sched_junk as $dt_str) {
					$perc_str = 'PERCENT_' . $known . $str . $dt_str;
					$cstate[$perc_str] =
						percent(get_array_var($cstate, $str . $dt_str), $div);
				}
			}

			$str = 'PERCENT_KNOWN_TIME_' . $state;
			$cstate[$str] =
				$cstate[$str . '_SCHEDULED'] + $cstate[$str . '_UNSCHEDULED'];
		}

		# mop up the oddballs and special cases
		$cstate['PERCENT_TIME_UNDETERMINED_NOT_RUNNING'] =
			percent($cstate['TIME_UNDETERMINED_NOT_RUNNING'], $div);
		$cstate['PERCENT_TIME_UNDETERMINED_NO_DATA'] =
			percent($cstate['TIME_UNDETERMINED_NO_DATA'], $div);

		return $cstate;
	}

	function st_update($end_time)
	{
		$prev_time = $this->st_prev_row['the_time'];
		$duration = $end_time - $prev_time;
		$active = intval($this->tp_active_time($prev_time, $end_time));
		$this->st_inactive += ($end_time - $prev_time) - $active;

		$st = "$this->st_running:$this->st_dt_depth:$this->st_obj_state";
		if ($active) {
			if (!isset($this->st_raw[$st]))
				$this->st_raw[$st] = $active;
			else
				$this->st_raw[$st] += $active;
		}

		if ($this->st_needs_log) {
			$this->st_prev_row['duration'] = $duration;

			$keep_log_line = true;

			if ($this->st_needs_log_maxrows > 0 && count($this->st_log) >= $this->st_needs_log_maxrows)
				$keep_log_line = false;

			# if this is a host or service event, check against the states_mask if we should keep it
			if (array_key_exists('host_name', $this->st_prev_row)) {
				$is_service = array_key_exists('service_description', $this->st_prev_row) && !empty($this->st_prev_row['service_description']);
				$log_state_mask = state_to_mask(! $is_service, $this->st_prev_row['state']);

				#var_dump($this->st_prev_row);
				#echo " is_service is " . $is_service;
				#echo " log_state_mask is " . $log_state_mask;
				#var_dump(mask_to_states_str($log_state_mask));
				#echo "<br>";
				#echo "<br>";
				#echo "<br>";

				$state_mask_match = ($log_state_mask & $this->st_needs_log_states_mask);
				if (! $state_mask_match) {
					$keep_log_line = false;
				}
				#echo "comparing log_mask " . $log_state_mask  . " to " . $this->st_needs_log_states_mask . " match: " . $state_mask_match . "<br>";
			}

			if ($keep_log_line) {
				#echo sprintf("adding log h:%s s:%s state:%s<br>", $this->st_prev_row['host_name'], $this->st_prev_row['service_description'], $this->st_prev_row['state']);
				$this->st_log[] = $this->st_prev_row;
			} else {
				#echo sprintf("not adding log %s %s %s<br>", $this->st_prev_row['host_name'], $this->st_prev_row['service_description'], $this->st_prev_row['state']);
			}
		}
	}

	/**
	 * Strictly for debugging purposes. Enable this if you get
	 * weird values from the reporting tool for some reason
	 */
	function check_st_sub_discrepancies($row = false)
	{
		$disc_desc = array();
		$disc = 0;
		$st_sub_totals = 0;

		if (!$this->sub_reports)
			return;

		foreach ($this->st_sub as $state => $cnt) {
			if ($cnt < 0)
				echo "WARN: $state count is $cnt\n";
			$st_sub_totals += $cnt;
		}

		$actual = $this->st_sub;
		$stash = array();
		foreach ($actual as $state => $cnt)
			$actual[$state] = 0;
		$real = $actual;
		foreach ($this->sub_reports as $rpt) {
			$actual[$rpt->st_obj_state]++;
			$real[$rpt->st_real_state]++;
			$stash[$rpt->st_obj_state][] = $rpt;
		}

		foreach ($actual as $state => $cnt) {
			if ($this->st_sub[$state] !== $cnt) {
				$disc_desc[] = "DISCREPANCY: ($state): actual=$cnt; st_sub=" . $this->st_sub[$state] . "\n";
				$disc++;
			}
		}
		if ($disc != $this->st_sub_discrepancies) {
			echo "Discrepancy change. Old=$this->st_sub_discrepancies; New=$disc\n";
			echo "Last row was: "; print_r($row);
			print_r($disc_desc);
			$src = $row['host_name'];
			foreach ($this->sub_reports as $rpt) {
				if ($rpt->st_source = $src) {
					echo "Current state of offending object: "; print_r($rpt);
					exit(1);
				}
			}
		}
		$this->st_sub_discrepancies = $disc;
	}

	function st_parse_row($row = false)
	{
		$this->register_db_time($row['the_time']);

		$this->st_update($row['the_time']);

		$obj_name = $sub = false;
		if (!empty($row['service_description'])) {
			$obj_name = $row['host_name'] . ";" . $row['service_description'];
		}
		elseif (!empty($row['host_name'])) {
			$obj_name = $row['host_name'];
		}

		# sub-reports must be be st_update()d before we
		# set its state in the case statement below
		$sub = false;
		if ($obj_name && isset($this->sub_reports[$obj_name])) {
			$sub = $this->sub_reports[$obj_name];
			$sub->st_update($row['the_time']);
		}

		# if we get an event and monitor is stopped, we
		# consider it started. This is necessary to prevent
		# us from generating bogus reports in case there's
		# a missing PROCESS_START event in the database
		$is_running = intval($row['event_type'] !== PROCESS_SHUTDOWN);

		# only update sub-reports on statechange
		if ($is_running != $this->st_running) {
			foreach ($this->sub_reports as $rpt)
				$rpt->st_running = $is_running;
			$this->st_running = $is_running;
		}

		switch($row['event_type']) {
		 case DOWNTIME_START:
			$row['output'] = $this->st_obj_type . ' has entered a period of scheduled downtime';
			# Due to a bug in the nagios module (not the import
			# program), DOWNTIME_START events sometimes have a
			# downtime_depth of 0 in the database (which is clearly
			# bogus). We work around it by making sure it's always
			# at least 1.
			if (!$row['downtime_depth'])
				$row['downtime_depth'] = 1;
			# For the same reason, we must save the dt depth for
			# when we encounter the DOWNTIME_STOP event later, so
			# we know which value it *should* have.
			$this->st_last_dt_start_depth[$obj_name] = $row['downtime_depth'];

			# update this sub-object's status to OK if we're supposed
			# to calculate scheduled downtime as uptime
			if ($this->scheduled_downtime_as_uptime && $sub && $sub->st_obj_state != STATE_OK) {
				$sub->st_dt_depth++;
				$sub->calculate_object_state();
				$sub->sub->st_obj_state = STATE_OK;
				$this->st_sub[$sub->st_real_state]--;
				$this->st_sub[$sub->st_obj_state]++;
			}
			if ($sub) {
				# sub->st_dt_depth must be set before we get the
				# common downtime depth here
				$sub->st_dt_depth = intval(!!$row['downtime_depth']);
				$this->st_dt_depth = $this->get_common_downtime_state();
			} else {
				$this->st_dt_depth = intval(!!$row['downtime_depth']);
			}
			break;
		 case DOWNTIME_STOP:
			$row['output'] = $this->st_obj_type . ' has exited a period of scheduled downtime';

			# Similar to the fix above, we check the downtime
			# depth of the last start event here to make sure
			# it's decremented by one (unless it's already zero).
			if ($row['downtime_depth']) {
				if (!isset($this->st_last_dt_start_depth[$obj_name]))
					$row['downtime_depth'] = 0;
				elseif ($row['downtime_depth'] == $this->st_last_dt_start_depth[$obj_name]) {
					$row['downtime_depth'] = --$this->st_last_dt_start_depth[$obj_name];
				}
			}

			if ($sub) {
				# sub->st_dt_depth must be set before we retrieve the
				# common downtime depth
				$sub->st_dt_depth = intval(!!$row['downtime_depth']);
				$this->st_dt_depth = $this->get_common_downtime_state();
			} else {
				$this->st_dt_depth = intval(!!$row['downtime_depth']);
			}

			# possibly restore the actual object state if
			# scheduled downtime ends
			if ($this->scheduled_downtime_as_uptime) {
				if ($sub && !$sub->st_dt_depth && $sub->st_obj_state != $sub->st_real_state) {
					$this->st_sub[$sub->st_obj_state]--;
					$this->st_sub[$sub->st_real_state]++;
					$sub->calculate_object_state();
				}
				else
					$this->calculate_object_state();
			}
			break;

		 case SERVICECHECK:
		 case HOSTCHECK:
			$state = $row['state'];
			if ($sub && $sub->scheduled_downtime_as_uptime && $sub->st_dt_depth)
				$state = STATE_OK;

			# update the real state of the object
			if ($sub)
				$sub->st_real_state = $row['state'];
			else
				$this->st_real_state = $row['state'];

			if ($sub && $sub->st_obj_state != $state) {
				$this->st_sub[$sub->st_obj_state]--;
				$this->st_sub[$state]++;
				$this->st_obj_state = $this->st_worst();
			}
			break;
		 default:
			//ERROR
		}

		if ($sub)
			$sub->calculate_object_state();

		$this->calculate_object_state();

		# fairly nifty debugging check. This is the place to
		# call it if you're going to.
		#$this->check_st_sub_discrepancies($row);

		# this must come after the case table above
		if ($sub) {
			$sub->st_prev_row = $row;
		}

		# we mangle $row here, since $this->st_prev_row is always
		# derived from it, except when it's the initial
		# state which always has (faked) output
		if (empty($row['output']))
			$row['output'] = '(No output)';

		$this->st_prev_row = $row;
	}

	function st_worst()
	{
		if (empty($this->sub_reports))
			return $this->st_obj_state;

		if ($this->st_is_service)
			return $this->get_worst_service_state();
		return $this->get_worst_host_state();
	}

	function calculate_object_state($state = false)
	{
		if ($this->sub_reports) {
			$this->st_obj_state = $this->st_worst();
			return;
		}

		if (!$state)
			$state = $this->st_real_state;

		# if we're counting scheduled downtime as uptime and
		# we're in downtime, the calculated state is always OK
		if ($this->scheduled_downtime_as_uptime && $this->st_dt_depth) {
			$this->st_obj_state = STATE_OK;
		} else {
			$this->st_obj_state = $state;
		}
	}

	/**
	 * @name	st_init
	 * @desc	Initialize the the state machine for this report
	 */
	function st_init($hostname = false, $servicename = false)
	{
		if (!$this->timeperiods_resolved)
			$this->resolve_timeperiods();

		if (!$this->master && empty($this->sub_reports)) {
			$this->st_needs_log = true;
		}
		if ($this->st_needs_log === true) {
			$this->st_log = array();
		}

		if (!empty($servicename)) {
			$this->st_is_service = true;
			$this->st_obj_type = 'Service';
			$this->st_source = $servicename;
			if (is_array($servicename))
				$this->id = $servicename;
		}
		else {
			# we need at least a service or a host
			if (empty($hostname))
				return false;
			if (is_array($hostname))
				$this->id = $hostname;
			$this->st_obj_type = 'Host';
			$this->st_source = $hostname;
		}

		if (is_array($this->st_source))
			$this->st_source = implode(',', $this->st_source);

		$host_state_txt 	= array(0 => 'UP', 1 => 'DOWN', 2 => 'UNREACHABLE');
		$service_state_txt 	= array(0 => 'OK', 1 => 'WARNING', 2 => 'CRITICAL', 3 => 'UNKNOWN');
		$this->st_text = empty($servicename) ? $host_state_txt : $service_state_txt;
		$this->st_text[STATE_PENDING] = 'PENDING';

		# id must always be set properly for single-object reports
		if (empty($this->id)) {
			$this->id = $hostname;

			if (!empty($servicename))
				$this->id .= ";$servicename";
		}

		# prime the state counter for sub-objects
		if (!empty($this->sub_reports)) {
			foreach ($this->st_text as $st => $discard)
				$this->st_sub[$st] = 0;
			foreach ($this->sub_reports as $rpt) {
				$rpt->st_dt_depth = $rpt->initial_dt_depth;
				$rpt->calculate_object_state();
				$this->st_sub[$rpt->st_obj_state]++;
			}
			$this->st_obj_state = $this->st_worst();
			$this->st_dt_depth = $this->get_common_downtime_state();
		}
		else {
			$this->st_dt_depth = intval(!!$this->get_initial_dt_depth($hostname, $servicename));
			$this->st_real_state = $this->get_initial_state($hostname, $servicename);
			$this->calculate_object_state($this->st_real_state);
		}

		# $last_shutdown is only set if monitor is stopped, so set
		# $this->st_running to its boolean inverse.
		$this->st_running = intval(!$this->last_shutdown);

		$fout = "";
		if (!$this->st_running)
			$fevent_type = PROCESS_SHUTDOWN;
		else {
			if ($this->st_is_service)
				$fevent_type = SERVICECHECK;
			else
				$fevent_type = HOSTCHECK;
		}
		$this->st_prev_row = array
			('state' => $this->st_obj_state,
			 'the_time' => $this->start_time,
			 'event_type' => $fevent_type,
			 'downtime_depth' => $this->st_dt_depth);

		# if we're actually going to use the log, we'll need
		# to generate a faked initial message for it.
		if ($this->st_needs_log) {
			$fout = sprintf("Report period start. Daemon is%s running, " .
			                "we're%s in scheduled downtime, state is %s (%d)",
			                $this->st_running ? '' : ' not',
			                $this->st_dt_depth ? '' : ' not',
			                $this->st_text[$this->st_real_state], $this->st_real_state);
			$this->st_prev_row['output'] = $fout;

			if (!empty($hostname) && is_string($hostname))
				$this->st_prev_row['host_name'] = $hostname;

			if (!empty($servicename) && is_string($servicename))
				$this->st_prev_row['service_description'] = $servicename;
		}
	}
	/**
	 *	@name 	calculate_uptime
	 *	@desc
	 * 	@param  str $hostname
	 * 	@param	str $servicename

	 * @global MysqlDB Database interface used.
	 * @global bool Whether to give verbose debugging.
	 * 	@return mixed Array or false on error:
	 * The array is of the form:<br>
	 * array(<br>
	 * 	'source' => string,<br>
	 * 	'log' => array,<br>
	 * 	'states' => array,<br>
	 * 	'tot_time' => int,<br>
	 * 	'groupname' => string<br>
	 * 	)<br>
	 */
	function calculate_uptime($hostname=false, $servicename=false)
	{
		$this->st_init($hostname, $servicename);

		$this->st_parse_all_rows($hostname, $servicename);
		return $this->st_finalize();
	}

	function st_parse_all_rows($hostname = false, $servicename = false)
	{
		global $dba;

		# run the query and loop through the result one by one
		$result = $this->uptime_query($hostname, $servicename);
		while ($row = $dba->getnext($result)) {
			$this->st_parse_row($row);
		} // end while
	}

	function st_finalize()
	{
		# gather remaining time. If they match, it'll be 0
		$this->st_update($this->end_time);

		$converted_state = $this->convert_state_table($this->st_raw, $this->st_text);

		if ($this->use_average && !empty($this->sub_reports) > 0) {
			$converted_state = $this->calculate_average();
		}

		# state template array depends on what we are checking
		$tpl = $this->state_tpl_host;
		if ($this->st_is_service)
			$tpl = $this->state_tpl_svc;
		foreach ($tpl as $t => $discard)
			if (!isset($converted_state[$t]))
				$converted_state[$t] = 0;

		if (empty($this->sub_reports)) {
			$converted_state['HOST_NAME'] = $this->id;
			if ($this->st_is_service) {
				$srv = explode(';', $this->id);
				$converted_state['HOST_NAME'] = $srv[0];
				$converted_state['SERVICE_DESCRIPTION'] = $srv[1];
			}
		} else {
			if ($this->st_is_service) {
				unset($converted_state['HOST_NAME']);
				$converted_state['SERVICE_DESCRIPTION'] = $this->id;
			}
			else
				$converted_state['HOST_NAME'] = $this->id;
		}

		# now add the time we didn't count due
		# to the selected timeperiod
		$converted_state['TIME_INACTIVE'] = $this->st_inactive;

		$this->states = $converted_state;
		$total_time = $this->end_time - $this->start_time;
		$groupname = $this->hostgroup!='' ? $this->hostgroup : $this->servicegroup;
		return array('source' => $this->st_source, 'log' => $this->st_log, 'log_maxrows' => $this->st_needs_log_maxrows, 'states' => $converted_state, 'tot_time' => $total_time, 'groupname' => $groupname);
	}

	/**
	 *	@name 	uptime_query
	 *	@desc 	Get log details for host/service
	 * 	@param 	str $hostname
	 * 	@param	str $servicename
	 * 	@return res resultset or false on error
	 */
	function uptime_query($hostname=false, $servicename=false)
	{
		global $dba, $sql_class;
		$event_type = HOSTCHECK;
		if ($servicename) {
			$event_type = SERVICECHECK;
			$servicename = $sql_class->escapestr($servicename);
		}

		# this query works out automatically, as we definitely don't
		# want to get all state entries for a hosts services when
		# we're only asking for uptime of the host
		$sql = "SELECT host_name, service_description, " .
			"state,timestamp AS the_time, hard, event_type, " .
			"downtime_depth";
		# output is a TEXT field, so it needs an extra disk
		# lookup to fetch and we don't (usually) need it
		if ($this->st_needs_log)
			$sql .= ", output";

		$sql .= " FROM ".$this->db_name.".".$this->db_table." " .
			"WHERE timestamp >=".$this->start_time." " .
			"AND timestamp <=".$this->end_time." ";
		if (is_array($hostname) && empty($servicename)) {
			$sql .= "AND (host_name IN ('" . join("', '", $hostname) . "') AND service_description = '') ";
		}
		elseif (is_array($servicename)) {
			$sql .= "AND concat(host_name, ';', service_description) IN ('" .
				join("', '", $servicename) . "') ";
		}
		else {
			$sql .= "AND host_name='".$hostname."' " .
			"AND service_description='".$servicename."' ";
		}
		if (!$this->include_soft_states)
			$sql .= 'AND hard = 1 ';
		$sql .= "AND ( " .
			"event_type=" . $event_type . ' ' .
			"OR event_type=" . DOWNTIME_START . ' ' .
			"OR event_type=" . DOWNTIME_STOP . ' ';
		if (!$this->assume_states_during_not_running)
			$sql .= "OR event_type=".PROCESS_SHUTDOWN.
			" OR event_type=".PROCESS_START;
		$sql .= ") ORDER BY id";

		return $dba->query($sql);
	}

	/**
	 *	@name 	get_initial_dt_depth
	 *	@desc 	Fetch information about SCHEDULED_DOWNTIME status
	 * 	@param 	str $hostname
	 * 	@param 	str $service_description
	 * 	@return int depth of initial downtime
	 *
	 */
	function get_initial_dt_depth($hostname=false, $service_description=false)
	{
		global $dba;

		if ($this->initial_dt_depth != false)
			return $this->initial_dt_depth;

		if (empty($hostname)) {
			return false;
		}

		$sql = "SELECT id, timestamp, event_type, downtime_depth FROM " .
			$this->db_name . "." . $this->db_table . " " .
			"WHERE timestamp <= " . $this->start_time . " AND " .
			"(event_type = " . DOWNTIME_START .
			" OR event_type = " . DOWNTIME_STOP . ") " .
			" AND host_name = '" . $hostname . "' ";

		if ($service_description!='') {
			$sql .= "AND service_description='".$service_description."' ";
		}
		$sql .= " ORDER BY id DESC LIMIT 1";

		if (!$res = $dba->execute($sql)) {
			if (!empty($dba->error)) {
				__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
				return false;
			}
			return 0;
		}

		$row = $res[0];
		$this->register_db_time($row['timestamp']);
		$this->initial_dt_depth = $row['downtime_depth'];
		return $this->initial_dt_depth;
	}

	/**
	 *	@name 	get_initial_state
	 *	@desc 	Get initital state from db.
	 * 			This is usually required when selecting states
	 * 			for a host/service when the selected start time
	 * 			don't exactly match a record in db.
	 * 	@return obj Record fr db or false on error
	 *
	 */
	function get_initial_state($host_name='', $service_description='')
	{
		global $dba;
		$state = false;

		if ($this->initial_state !== false)
			return $this->initial_state;

		// we always need timestamp and at least a host_name
		if (empty($host_name)) {
			return false;
		}

		$sql = "SELECT timestamp, state FROM " .
			$this->db_name . "." . $this->db_table .
			" WHERE host_name = '$host_name' " .
			" AND service_description = '$service_description' " .
			" AND event_type = ";

		if ($service_description !='' ) {
			$assumed_state = $this->initial_assumed_service_state;
			$sql .= SERVICECHECK;
		} else {
			$assumed_state = $this->initial_assumed_host_state;
			$sql .= HOSTCHECK;
		}
		if (!$this->include_soft_states)
			$sql .= ' AND hard = 1';

		$sql .= ' ';
		$base_sql = $sql;
		$sql .= "AND timestamp < " . $this->start_time .
			" ORDER BY id DESC LIMIT 1";

		if (!($res = $dba->execute($sql))) {
			if (!empty($dba->error)) {
				__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
				return false;
			}

			# There is no real initial state, so check if we should
			# assume something. If it's a real state, return early
			if ($assumed_state > 0) {
				$this->initial_state = $assumed_state;
				return $this->initial_state;
			}

			$state = $assumed_state;
			# state == -1 is magic for "use current state as initial"
			# it's fairly bonkers to do that, and will yield different
			# results for historical data based on present state, but
			# it's supported in the old cgi's, so we must keep this
			# mouldering wreck of insanity alive...
			if ($state == -1) {
				$sql = $base_sql . "ORDER BY id DESC LIMIT 1";
				$res = $dba->execute($sql);
			}

			# Using the first real state found in the database as
			# the assumed initial state is a lot less evil than the
			# above black voodoo, as reports for last year will always
			# look the same, no matter what the current state is.
			elseif ($state == -3) {
				$sql = $base_sql . "ORDER BY id ASC LIMIT 1";
				$res = $dba->execute($sql);
			}

			if (!($res = $dba->execute($sql))) {
				if (!empty($dba->error)) {
					__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
					return false;
				}

				# this is only reached if there is no state at all
				# in the database. It should usually be an error,
				# unless one tries to take a report from, say, last
				# year on a host that was added less than 30 seconds
				# ago. Either way, we're out of options so do nothing.
				# $state will default to STATE_PENDING further down
			}
		}
		/* state assumption logic end */

		if (!empty($res))
			$state = $res[0]['state'];

		if ($state === false || $state < 0)
			$state = STATE_PENDING;

		$this->initial_state = $state;
		return $state;
	}

	/**
	 *	@name 	datetime_to_timestamp
	 *	@desc 	Convert between datetime in format
	 * 			YYYY-MM-DD H:i:s to UNIX TIMESTAMP
	 *
	 */
	function datetime_to_timestamp($datetime='')
	{
		global $dba;
		$datetime = trim($datetime);

		if (is_numeric($datetime))
			return intval($datetime);

		if (!strstr($datetime, '-')) {
			// not fully numeric and no dash; wrong format
			return false;
		}

		$sql = "SELECT UNIX_TIMESTAMP('".$datetime."') AS timestamp";
		if (!$res = $dba->execute($sql)) {
			if (!empty($dba->error))
				__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
			return false;
		}
		if (empty($res))
			return false;
		return $res[0]['timestamp'];
	}

	function get_common_downtime_state()
	{
		if (!$this->sub_reports)
			return $this->st_dt_depth;

		foreach ($this->sub_reports as $rpt) {
			if (!$rpt->st_dt_depth)
				return 0;
		}

		return 1;
	}

	function get_worst_host_state()
	{
		if (!empty($this->st_sub[HOST_DOWN]))
			return HOST_DOWN;
		if (!empty($this->st_sub[HOST_UNREACHABLE]))
			return HOST_UNREACHABLE;
		if (!empty($this->st_sub[HOST_PENDING]) && empty($this->st_sub[HOST_UP]))
			return HOST_PENDING;
		return  HOST_UP;
	}

	function get_worst_service_state()
	{
		if (!empty($this->st_sub[SERVICE_CRITICAL]))
			return SERVICE_CRITICAL;
		if (!empty($this->st_sub[SERVICE_WARNING]))
			return SERVICE_WARNING;
		if (!empty($this->st_sub[SERVICE_UNKNOWN]))
			return SERVICE_UNKNOWN;

		if (!empty($this->st_sub[SERVICE_PENDING]) && empty($this->st_sub[SERVICE_OK]))
			return SERVICE_PENDING;

		return SERVICE_OK;
	}

	/**
	 * Calculate average values using every subreport
	 * @return array Average values for the group
	 */
	protected function calculate_average()
	{
		if (empty($this->sub_reports)) {
			return;
		}

		$ret = array();
		$num_subs = 0;
		foreach($this->sub_reports as $report) {
			$num_subs++;
			foreach ($report->states as $k => $v) {
				if (!isset($ret[$k]))
					$ret[$k] = 0;

				$ret[$k] += $v;
			}
		}

		foreach ($ret as $k => $v) {
			$ret[$k] = $v / $num_subs;
		}

		return $ret;
	}
	/**
	 * Parses given input as a nagios 3 timeperiod variable. If valid, it is added to the report
	 * Code is derived from the nagios 3 sources (xdata/xodtemplate.c)
	 *
	 * @param str $name
	 * @param str $value
	 * @return boolean
	 *
	 * FIXME: find better way of adding 24h to end date
	 */
	public function set_timeperiod_variable($name, $value)
	{
		# from timeperiod_class.php
		global $valid_weekdays, $valid_months;

		$weekday_numbers = array_flip($valid_weekdays);
		$month_numbers = array_flip($valid_months);

		if(in_array($name, $valid_weekdays)) # add regular weekday include time
		{
			return $this->set_option($name, $value);
		}

		$input = "$name $value";

		# you could put this in one line but that will be too messy
		$items = array_filter(sscanf($input,"%4d-%2d-%2d - %4d-%2d-%2d / %d %[0-9:, -]"));
		if(count($items) == 8)
		{
			list($syear, $smon, $smday, $eyear, $emon, $emday, $skip_interval, $timeranges) = $items;

			/* add timerange exception */
			$this->add_timeperiod_exception(Timeperiod::DATERANGE_CALENDAR_DATE,
				$syear, $smon, $smday, 0, 0, $eyear, $emon, $emday, 0, 0, $skip_interval, $timeranges);
			return true;
		}

		$items = array_filter(sscanf($input,"%4d-%2d-%2d / %d %[0-9:, -]"));
		if(count($items) == 5)
		{
			list($syear,$smon, $smday, $skip_interval, $timeranges) = $items;
			$eyear = $syear;
			$emon  = $smon;
			$emday = $smday;

			/* add timerange exception */
			$this->add_timeperiod_exception(Timeperiod::DATERANGE_CALENDAR_DATE,
				$syear, $smon, $smday, 0, 0, $eyear, $emon, $emday, 0, 0, $skip_interval, $timeranges);
			return true;
		}

		$items = array_filter(sscanf($input,"%4d-%2d-%2d - %4d-%2d-%2d %[0-9:, -]"));
		if(count($items) == 7)
		{
			list($syear, $smon, $smday, $eyear, $emon, $emday, $timeranges) = $items;

			/* add timerange exception */
			$this->add_timeperiod_exception(Timeperiod::DATERANGE_CALENDAR_DATE,
				$syear, $smon, $smday, 0, 0, $eyear, $emon, $emday, 0, 0, 0, $timeranges);
			return true;
		}

		$items=array_filter(sscanf($input,"%4d-%2d-%2d %[0-9:, -]"));
		if(count($items)==4)
		{
			list($syear, $smon, $smday, $timeranges) = $items;
			$eyear = $syear;
			$emon = $smon;
			$emday = $smday;
			/* add timerange exception */
			$this->add_timeperiod_exception(Timeperiod::DATERANGE_CALENDAR_DATE,
				$syear, $smon, $smday, 0, 0, $eyear, $emon, $emday, 0, 0, 0, $timeranges);
			return true;
		}

		/* other types... */
		$items = array_filter(sscanf($input,"%[a-z] %d %[a-z] - %[a-z] %d %[a-z] / %d %[0-9:, -]"));
		if(count($items) == 8)
		{
			list($str1, $swday_offset, $str2, $str3, $ewday_offset, $str4, $skip_interval, $timeranges) = $items;
			/* wednesday 1 january - thursday 2 july / 3 */

			if(in_array($str1, $valid_weekdays) &&
				in_array($str2, $valid_months) &&
				in_array($str3, $valid_weekdays) &&
				in_array($str4, $valid_months))
			{
				$swday = $weekday_numbers[$str1];
				$smon = $month_numbers[$str2];
				$ewday = $weekday_numbers[$str3];
				$emon = $month_numbers[$str4];

				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_WEEK_DAY,
					0, $smon, 0, $swday, $swday_offset, 0, $emon, 0, $ewday, $ewday_offset, $skip_interval,  $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d - %[a-z] %d / %d %[0-9:, -]"));
		if(count($items) == 6)
		{
			list($str1, $smday, $str2, $emday, $skip_interval, $timeranges) = $items;
			/* february 1 - march 15 / 3 */
			/* monday 2 - thursday 3 / 2 */
			/* day 4 - day 6 / 2 */
			if(in_array($str1, $valid_weekdays) && in_array($str2, $valid_weekdays))
			{
				/* monday 2 - thursday 3 / 2 */
				$swday = $weekday_numbers[$str1];
				$ewday = $weekday_numbers[$str2];
				$swday_offset = $smday;
				$ewday_offset = $emday;

				/* add timeperiod exception */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_WEEK_DAY,
					0, 0, 0, $swday, $swday_offset, 0, 0, 0, $ewday, $ewday_offset, $skip_interval, $timeranges);
				return true;
			}
			elseif(in_array($str1, $valid_months) && in_array($str2, $valid_months))
			{
				$smon = $month_numbers[$str1];
				$emon = $month_numbers[$str2];
				/* february 1 - march 15 / 3 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DATE,
					0, $smon, $smday, 0, 0,
					0, $emon, $emday, 0, 0, $skip_interval, $timeranges);
				return true;
			}
			else if(!strcmp($str1,"day")  && !strcmp($str2,"day"))
			{
				/* day 4 - 6 / 2 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DAY,
					0, 0, $smday, 0, 0, 0, 0, $emday, 0, 0, $skip_interval, $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d - %d / %d %[0-9:, -]"));
		if(count($items) == 5)
		{
			list($str1, $smday, $emday, $skip_interval, $timeranges) = $items;

			/* february 1 - 15 / 3 */
			/* monday 2 - 3 / 2 */
			/* day 1 - 25 / 4 */
			if(in_array($str1, $valid_weekdays))
			{
				$swday = $weekday_numbers[$str1];
				/* thursday 2 - 4 */
				$swday_offset = $smday;
				$ewday = $swday;
				$ewday_offset = $emday;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_WEEK_DAY,
					0, 0, 0, $swday, $swday_offset, 0, 0, 0, $ewday, $ewday_offset, $skip_interval, $timeranges);
				return true;
			}
			else if(in_array($str1, $valid_months))
			{
				$smon = $month_numbers[$str1];
				$emon = $smon;
				/* february 3 - 5 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DATE,
					0, $smon, $smday, 0, 0,
					0, $emon, $emday, 0, 0, $skip_interval, $timeranges);
				return true;
			}
			else if(!strcmp($str1, "day"))
			{
				/* day 1 - 4 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DAY,
					0, 0, $smday, 0, 0, 0, 0, $emday, 0, 0, $skip_interval, $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d %[a-z] - %[a-z] %d %[a-z] %[0-9:, -]"));
		if(count($items)  == 7)
		{
			list($str1, $swday_offset, $str2, $str3, $ewday_offset, $str4, $timeranges) = $items;

			/* wednesday 1 january - thursday 2 july */
			if(in_array($str1, $valid_weekdays) && in_array($str2, $valid_months) &&
				in_array($str3, $valid_weekdays) && in_array($str4, $valid_months))
			{
				$swday = $weekday_numbers[$str1];
				$smon = $month_numbers[$str2];
				$ewday = $weekday_numbers[$str3];
				$emon = $month_numbers[$str4];
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_WEEK_DAY,
					0, $smon, 0, $swday, $swday_offset, 0, $emon, 0, $ewday, $ewday_offset, 0, $timeranges);
				return true;
			}
			return false;
		}

		$items=array_filter(sscanf($input,"%[a-z] %d - %d %[0-9:, -]"));
		if(count($items) == 4)
		{
			list($str1, $smday, $emday, $timeranges) = $items;

			/* february 3 - 5 */
			/* thursday 2 - 4 */
			/* day 1 - 4 */
			if(in_array($str1, $valid_weekdays))
			{
				/* thursday 2 - 4 */
				$swday = $weekday_numbers[$str1];
				$swday_offset = $smday;
				$ewday = $weekday_numbers[$swday];
				$ewday_offset = $emday;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_WEEK_DAY,
					0, 0, 0, $swday, $swday_offset, 0, 0, 0, $ewday, $ewday_offset, 0, $timeranges);
				return true;
			}
			else if(in_array($str1, $valid_months))
			{
				/* february 3 - 5 */
				$smon = $month_numbers[$str1];
				$emon = $smon;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DATE,
					0, $smon, $smday, 0, 0, 0, $emon, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			else if(!strcmp($str1,"day"))
			{
				/* day 1 - 4 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DAY,
					0, 0, $smday, 0, 0, 0, 0, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d - %[a-z] %d %[0-9:, -]"));
		 if(count($items) == 5)
		{
			list($str1, $smday, $str2, $emday, $timeranges) = $items;
			/* february 1 - march 15 */
			/* monday 2 - thursday 3 */
			/* day 1 - day 5 */
			if(in_array($str1, $valid_weekdays) && in_array($str2, $valid_weekdays))
			{
				/* monday 2 - thursday 3 */
				$swday = $weekday_numbers[$str1];
				$ewday = $weekday_numbers[$str2];
				$swday_offset = $smday;
				$ewday_offset = $emday;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_WEEK_DAY,
					0, 0, 0, $swday, $swday_offset, 0, 0, 0, $ewday, $ewday_offset, 0, $timeranges);
				return true;
			}
			elseif(in_array($str1, $valid_months) && in_array($str2, $valid_months))
			{
				/* february 1 - march 15 */
				$smon = $month_numbers[$str1];
				$emon = $month_numbers[$str2];
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DATE,
					0, $smon, $smday, 0, 0, 0, $emon, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			else if(!strcmp($str1,"day")  && !strcmp($str2,"day"))
			{
				/* day 1 - day 5 */
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DAY,
					0, 0, $smday, 0, 0, 0, 0, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d%*[ \t]%[0-9:, -]"));
		if(count($items) == 3)
		{
			list($str1, $smday, $timeranges) = $items;
			/* february 3 */
			/* thursday 2 */
			/* day 1 */
			if(in_array($str1, $valid_weekdays))
			{
				/* thursday 2 */
				$swday = $weekday_numbers[$str1];
				$swday_offset = $smday;
				$ewday = $swday;
				$ewday_offset = $swday_offset;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_WEEK_DAY,
					0, 0, 0, $swday, $swday_offset, 0, 0, 0, $ewday, $ewday_offset, 0, $timeranges);
				return true;
			}
			elseif(in_array($str1, $valid_months))
			{
				/* february 3 */
				$smon = $month_numbers[$str1];
				$emon = $smon;
				$emday = $smday;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DATE,
					0, $smon, $smday, 0, 0, 0, $emon, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			else if(!strcmp($str1,"day"))
			{
				/* day 1 */
				$emday = $smday;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_DAY,
					0, 0, $smday, 0, 0, 0, 0, $emday, 0, 0, 0, $timeranges);
				return true;
			}
			return false;
		}

		$items = array_filter(sscanf($input,"%[a-z] %d %[a-z] %[0-9:, -]"));
		if(count($items) == 4)
		{
			list($str1, $swday_offset, $str2, $timeranges) = $items;

			/* thursday 3 february */
			if(in_array($str1, $valid_weekdays) && in_array($str2, $valid_months))
			{
				$swday = $weekday_numbers[$str1];
				$smon = $month_numbers[$str2];
				$emon = $smon;
				$ewday = $swday;
				$ewday_offset = $swday_offset;
				$this->add_timeperiod_exception(Timeperiod::DATERANGE_MONTH_WEEK_DAY,
					0, $smon, 0, $swday, $swday_offset, 0, $emon, 0, $ewday, $ewday_offset, 0, $timeranges);
				return true;
			}
			// return false;
		}

		# syntactically incorrect variable
		return false;
	}

} /* class report_class */


/**
 *	@name get_hostgroup_members
 *	@desc
 *
 */
function get_hostgroup_members($hostgroup_name=false)
{
	global $dba;
	$hostgroup_name = trim($hostgroup_name);
	if (empty($hostgroup_name)) {
		return false;
	}
	$sql ="
		SELECT
			h.host_name,
			h.alias,
			h.address,
			hg.hostgroup_name,
			hg.alias AS hostgroup_alias
		FROM
			host h,
			hostgroup hg,
			host_hostgroup hhg
		WHERE
			hg.hostgroup_name='".$hostgroup_name."' AND
			hhg.hostgroup=hg.id AND
			h.id=hhg.host
		ORDER BY
			h.host_name,
			h.alias";

	if (!$res = $dba->execute($sql)) {
		if (!empty($dba->error))
			__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
		return false;
	}
	if (empty($res))
		return false;

	return $res;
}

/**
 *	@name 	get_servicegroup_members
 *	@desc
 *
 */
function get_servicegroup_members($servicegroup_name=false)
{
	global $dba;
	$servicegroup_name = trim($servicegroup_name);
	if (empty($servicegroup_name)) {
		return false;
	}
	$sql ="
		SELECT
			s.service_description,
			sg.servicegroup_name,
			sg.alias AS servicegroup_alias,
			h.host_name,
			h.alias,
			h.address
		FROM
			service s,
			servicegroup sg,
			service_servicegroup ssg,
			host h
		WHERE
			sg.servicegroup_name='".$servicegroup_name."' AND
			ssg.servicegroup=sg.id AND
			s.id=ssg.service AND
			h.id=s.host_name
		ORDER BY
			s.service_description";

	if (!$res = $dba->execute($sql)) {
		if (!empty($dba->error))
			__debug(__FUNCTION__.": line: ".__LINE__, $dba->error, $sql);
		return false;
	}
	if (empty($res))
		return false;

	return $res;
}

/**
 *	@name 	__debug
 *	@desc 	Debug message wrapper
 *
 */
function __debug($func='', $msg='', $sql='')
{
	$msg 	= trim($msg);
	$sql 	= trim($sql);
	$func 	= trim($func);
	if (DEBUG) {
		echo "<hr>DEBUG: <br />\n";
		if (!empty($func)) echo "FNC: ".$func."<br />\n";
		if (!empty($msg)) echo "MSG: ".$msg."<br />\n";
		if (!empty($sql)) echo "SQL: ".$sql."<br />\n";
		echo "<hr>";
	}
}

function percent($dividend, $divisor)
{
	if (!$dividend || !$divisor)
		return 0;

#	return sprintf("%.f", ($dividend / $divisor) * 100);
	return ($dividend / $divisor) * 100;
}

# from chart.php

function create_piechart($in_data=false)
{
	if (empty($in_data)) return false;

	$data = array_values($in_data);

	// A new graph
	$graph = new PieGraph(242,280,'auto');
	//$graph = new PieGraph(302,340,'auto');
	//$graph->SetShadow();

	// Turn on antialiasing
	$graph->SetAntiAliasing();

	// Setup title
	//$graph->title->Set("Pie plot by op5");
	//$graph->subtitle->Set('(With hidden 0 labels)');
	$graph->title->SetFont(FF_TREBUCHE);

	// The pie plot
	$p1 = new PiePlot($data);

	// Move center of pie
	//$p1->SetCenter(0.5,0.3);
	$p1->SetCenter(0.5,0.35);

	// uncomment to show guide markers for % value - slice
	//$p1->SetGuideLines(true, true, true);
	// No border
	$p1->ShowBorder(true, true);
	//$p1->SetBackgroundColor('red');
	//$graph->ShowBorder(true,true);

	// Label font and color setup
	$p1->value->SetFont(FF_ARIAL, FS_NORMAL, 8);
	$p1->value->SetColor('#000000');


	// Use absolute values (type==1)
	$p1->SetLabelType(PIE_VALUE_PER);

	// Label format
	$p1->value->SetFormat('%.3f%%');
	$p1->value->HideZero();
	$p1->value->Show();

	// Size of pie in fraction of the width of the graph
	$p1->SetSize(0.28);

	// Explode all slices
	//$p1->ExplodeAll(1);

	// Explode specific slice
	//$p1->ExplodeSlice(2);

	// Legends
	$p1->SetLegends(array_keys($in_data));
	$p1->SetSliceColors(get_color_values(array_keys($in_data)));

	// legend position
	$graph->legend->Pos(0.28,0.70);

	// remove legend border
	$graph->legend->SetFrameWeight(0);

	// remove legend shadow
	$graph->legend->SetShadow(0);
	//$graph->legend->SetFont(FF_ARIAL, FS_NORMAL, 8);
	// set legend bg color to white to hide it
	$graph->legend->SetFillColor('#ffffff');

	// remove outer image border
	$graph->SetFrame(false);

	$graph->Add($p1);
	$graph->Stroke();
}

function create_sla_piechart($in_data=false)
{
	global $colors;
	if (empty($in_data)) return false;

	$data = array_values($in_data);

	// A new graph
	$graph = new PieGraph(242,280,'auto');
	//$graph = new PieGraph(302,340,'auto');
	//$graph->SetShadow();

	// Turn on antialiasing
	$graph->SetAntiAliasing();

	// Setup title
	//$graph->title->Set("Pie plot by op5");
	//$graph->subtitle->Set('(With hidden 0 labels)');
	$graph->title->SetFont(FF_TREBUCHE);

	// The pie plot
	$p1 = new PiePlot($data);

	// Move center of pie
	//$p1->SetCenter(0.5,0.3);
	$p1->SetCenter(0.5,0.35);

	// uncomment to show guide markers for % value - slice
	//$p1->SetGuideLines(true, true, true);
	// No border
	$p1->ShowBorder(true, true);
	//$p1->SetBackgroundColor('red');
	//$graph->ShowBorder(true,true);

	// Label font and color setup
	$p1->value->SetFont(FF_ARIAL, FS_NORMAL, 8);
	$p1->value->SetColor('#000000');

	// Use absolute values (type==1)
	$p1->SetLabelType(PIE_VALUE_PER);

	// Label format
	$p1->value->SetFormat('%.1f%%');
	$p1->value->HideZero();
	$p1->value->Show();

	// Size of pie in fraction of the width of the graph
	$p1->SetSize(0.28);

	// Explode all slices
	//$p1->ExplodeAll(1);

	// Explode specific slice
	//$p1->ExplodeSlice(2);

	// Legends
#	$p1->SetLegends(array_keys($in_data));
	$graph->legend->Add('Percent Breached SLA', convert_hex_color($colors['red']));
	$graph->legend->Add('Percent Fulfilled SLA', convert_hex_color($colors['grey']));

	$p1->SetSliceColors(array('red', 'gray'));

	// legend position
	$graph->legend->Pos(0.28,0.70);

	// remove legend border
	$graph->legend->SetFrameWeight(0);

	// remove legend shadow
	$graph->legend->SetShadow(0);
	//$graph->legend->SetFont(FF_ARIAL, FS_NORMAL, 8);
	// set legend bg color to white to hide it
	$graph->legend->SetFillColor('#ffffff');

	// remove outer image border
	$graph->SetFrame(false);

	$graph->Add($p1);
	$graph->Stroke();
}

/**
 * @desc Assigns color to labels to be used in a piechart
 */
function get_color_values($labels=false)
{
	if (empty($labels)) return false;
	$green 	= '#88cd18';
	$yellow 	= '#ffd92f';
	$orange 	= '#ff9d08';
	$red 		= '#f7261b';
	$grey 	= '#a0a084';

	$return = false;
	$colors = array(
		'OK' => $green, 'UP' => $green, 'WARNING' => $yellow,
		'UNREACHABLE' => $orange, 'UNKNOWN' => $orange,
		'DOWN' => $red, 'CRITICAL' => $red,
		'UNDETERMINED' => $grey
	);
	foreach ($labels as $key) {
		$return[] = $colors[strtoupper($key)];
	}
	return $return;
}

/**
*	@name create_sla_bar_chart
*
*/
function create_sla_bar_chart($in_data=false)
{
	global $colors;
	if (empty($in_data)) return false;
	$real_values 	= false;
	$sla_values 	= false;
	$datax			= false;
	$fill_colors	= false;
	$set_colors		= false;
	$min_level		= 100;
	foreach ($in_data as $key => $values) {
		$real_values[] 	= $values[0];
		$sla_values[] 	= $values[1];
		$datax[] 		= ucfirst($key);
		$fill_colors[]	= $values[1] > $values[0] ?
			convert_hex_color($colors['red']) :
			convert_hex_color($colors['green']);
		$set_colors[]	= $values[1] > $values[0] ?
			convert_hex_color($colors['red']) :
			convert_hex_color($colors['green']);

		// Get the lowest SLA value
		$min_level = min($min_level, $values[1]);
	}

	// Create the graph.
	//$graph = new Graph(600,400,"auto");
	$graph = new Graph(804,400,"auto");
	$graph->SetScale("textlin");
	$graph->SetColor('#f4f4ed');

	$graph->img->SetMargin(50,20,35,60);
	$graph->SetFrame(true,'#f4f4ed',1);
	$graph->legend->SetFillColor('#f4f4ed');
	$graph->SetMarginColor("#f4f4ed");

	$graph->legend->Add('Fulfilled SLA', convert_hex_color($colors['green']));
	$graph->legend->Add('Breached SLA', convert_hex_color($colors['red']));
	$graph->legend->Add('SLA Compliance', convert_hex_color($colors['lightblue']));
	$graph->legend->SetLayout(LEGEND_HOR);
	$graph->legend->SetFont(FF_FONT1,FS_BOLD);
	$graph->legend->SetShadow(false);
	$graph->legend->SetFrameWeight(false);
	$graph->legend->Pos(0.285,0.93);

	// Create the linear error plot
	$l1plot=new LinePlot($sla_values);
	$l1plot->SetColor("blue");
	$l1plot->SetWeight(2);

	$l1plot->SetBarCenter();
	$l1plot->SetFillColor('skyblue@0.9');
	$l1plot->SetColor('navy@0.3');
	$l1plot->SetStyle('dashed');
	$l1plot->mark->SetType(MARK_DIAMOND);

	// Create the bar plot
	$bplot = new BarPlot($real_values);
	$bplot->value->SetFormat('%.3f');
	$bplot->value->Show();
	$bplot->SetValuePos('bottom');
	$bplot->SetWidth(0.7);

	$bplot->SetFillColor($fill_colors);

	// Make the bar graph start from something other than 0
	$bplot->SetYMin($min_level - ($min_level*0.05));

	// Add the plots to the graph
	$graph->Add($bplot);
	$graph->Add($l1plot);

	$graph->title->Set("Service Level Agreement");
	$graph->xaxis->title->Set("Period");
	$graph->yaxis->Settitle("Percent (%)", 'high');
	$graph->yaxis->SetTitlemargin(33);

	$graph->title->SetFont(FF_FONT1,FS_BOLD);
	$graph->yaxis->title->SetFont(FF_FONT1,FS_BOLD);
	$graph->xaxis->title->SetFont(FF_FONT1,FS_BOLD);

	$graph->xaxis->SetTickLabels($datax);

	// Display the graph
	$graph->Stroke();
}

/**
*	@name convert_hex_color
*	@desc Converts an html hexadecimal color from the form "#rrggbb" to its separate components
*  @param string $color_str Color represented as hexadecimal html representation
*  @return array The RGB components as a triplet with decimal values from 0 to 255.
*
*/
function convert_hex_color($color_str)
{
	if (empty($color_str)) return false;
	$color_str = str_replace('#', '', $color_str);
	$c1 = substr($color_str, 0, 2);
	$c2 = substr($color_str, 2, 2);
	$c3 = substr($color_str, 4, 2);
	return array(hexdec($c1), hexdec($c2), hexdec($c3) );
}

/**
 * Converts a date into timestamp, with some extra features such as
 * negative days of month to use days from the end of the month.
 * As for time, 00:00:00 of the day is used.
 *
 * @param int $year
 * @param int $month
 * @param int $monthday - Day of month, can be negative.
 * @return int The resulting timestamp.
 */
function calculate_time_from_day_of_month($year, $month, $monthday)
{
	$day = 0;

	/* positive day (3rd day) */
	if($monthday > 0)
	{
		$midnight = mktime(0,0,0, $month, $monthday, $year);

		/* if we rolled over to the next month, time is invalid */
		/* assume the user's intention is to keep it in the current month */
		if(date("n", $midnight) != $month)
			$midnight = 0;
	}
	else /* negative offset (last day, 3rd to last day) */
	{
		/* find last day in the month */
		$day = 32;
		do
		{
			/* back up a day */
			$day--;

			/* make the new time */
			$midnight = mktime(0,0,0, $month, $day, $year);

		} while(date("n", $midnight) != $month);

		/* now that we know the last day, back up more */

		/* -1 means last day of month, so add one to to make this correct - Mike Bird */
		$d = date("d", $midnight) + (($monthday < -30) ? -30 : $monthday + 1);
		$midnight = mktime(0,0,0, $month, $d, $year);

		/* if we rolled over to the previous month, time is invalid */
		/* assume the user's intention is to keep it in the current month */
		if(date("n", $midnight) != $month)
			$midnight = 0;
		}

	return $midnight;
}

function calculate_time_from_weekday_of_month($year, $month, $weekday, $weekday_offset)
{
	$weeks = 0;
	$midnight = mktime(0,0,0, $month, 1, $year);
	/* how many days must we advance to reach the first instance of the weekday this month? */
	$days = $weekday - date("w", $midnight);
	if($days < 0)
		$days += 7;
	/* positive offset (3rd thursday) */
	if($weekday_offset > 0)
	{
		/* how many weeks must we advance (no more than 5 possible) */
		$weeks = ($weekday_offset > 5) ? 5 : $weekday_offset;
		$days += (($weeks - 1) * 7);

		/* make the new time */
		$midnight = mktime(0,0,0, $month, $days + 1, $year);
		/* if we rolled over to the next month, time is invalid */
		/* assume the user's intention is to keep it in the current month */
		if(date("n", $midnight) != $month)
			$midnight = 0;
	}
	else 	/* negative offset (last thursday, 3rd to last tuesday) */
	{
		/* find last instance of weekday in the month */
		$days += (5*7);
		do
		{
			/* back up a week */
			$days -= 7;

			/* make the new time */
			$midnight = mktime(0,0,0, $month, $days + 1, $year);

			} while(date("n", $midnight) != $month);

		/* now that we know the last instance of the weekday, back up more */
		$weeks = ($weekday_offset < -5) ? -5 : $weekday_offset;
		$days = (($weeks + 1) * 7);

		$midnight = mktime(0,0,0, $month, $days + date("d", $midnight), $year);

		/* if we rolled over to the previous month, time is invalid */
		/* assume the user's intention is to keep it in the current month */
		if(date("n", $midnight) != $month)
			$midnight = 0;
		}
	return $midnight;
}

/**
 * Determines if two timeranges overlap
 * Note: stop time equal to start time in other range is NOT considered an overlap
 *
 * @param array $range1 Structure: array('start'=> {timestamp}, 'stop' => {timestamp})
 * @param array $range2 Structure: array('start'=> {timestamp}, 'stop' => {timestamp})
 * @param boolean $inclusive Whether to count "straddling" periods as ovelapping,
 * 	(Eg: start1 == stop2 or start2 == stop1)
 * @return boolean
 */
function timeranges_overlap(&$range1, &$range2, $inclusive=false)
{
	if($inclusive)
		return ($range1['start'] <= $range2['stop'] && $range2['start'] <= $range1['stop']) ||
	           ($range2['start'] <= $range1['stop'] && $range1['start'] <= $range2['stop']);

	return ($range1['start'] < $range2['stop'] && $range2['start'] < $range1['stop']) ||
	       ($range2['start'] < $range1['stop'] && $range1['start'] < $range2['stop']);
}

/**
 * Merges two timeranges into one.
 *
 * Assumes timeranges actually overlap and timeranges are correct (stop time after start time)
 *
 * @param array $src_range
 * @param array $dst_range
 * @return array
 */
function merge_timeranges(&$src_range, &$dst_range)
{
	return array('start' => min($src_range['start'], $dst_range['start']),
	             'stop'  => max($src_range['stop'],  $dst_range['stop']));
}
/**
 * Subtract one timerange from another
 *
 * Assumes proper timeranges
 *
 * @param array $timerange
 * @param array $subrange
 * @return array
 */
function subtract_timerange(&$timerange, &$subrange)
{
	$start = $timerange['start'];
	$stop  = $timerange['stop'];

	$sub_start = $subrange['start'];
	$sub_stop  = $subrange['stop'];

	if($sub_start > $start && $sub_stop < $stop && $sub_start < $sub_stop)
	{
		return array
		(
			array('start' => $start, 'stop' => $sub_start),
			array('start' => $sub_stop, 'stop' => $stop)
		);
	}


	if($sub_stop > $start && $sub_stop < $stop)
		$stop = $sub_stop;

	if($sub_start < $stop && $sub_start > $start)
		$start = $sub_start;

	return array(array('start' => $start, 'stop' => $stop));
}
/**
 * Add a new timerange to a set of timeranges.
 * If new range overlaps an existing range, the two are merged to one.
 *
 * Assumes timeranges contain only valid values (eg: stop time after start time)
 * Assumes the timerange set does not contain overlapping periods itself
 *
 * @param array $range
 * @param array $timerange_set
 *
 */
function add_timerange_to_set($range, &$timerange_set)
{
	for($i=0 ; $i<count($timerange_set) ; $i++)
	{
		$testrange = $timerange_set[$i];
		if(timeranges_overlap($range, $testrange, true))
		{
			# if range overlaps with current item, merge them and continue
			$range = merge_timeranges($range, $testrange);

			# remove the existing range, to later re-add it in the end
			unset($timerange_set[$i]);

			# to get the numerical indices back into sequence:
			$timerange_set = array_values($timerange_set);

			# Restart so we don't miss any element
			$i = 0;
		}
	}
	$timerange_set[] = $range;

	# recombobulate the indices
	$timerange_set = array_values($timerange_set);
}

/**
 * Merge two sets of timeranges into one, with no overlapping ranges.
 *
 * @param array $ranges1 (of structure array( array('start' => 1203120, 'stop' => 120399), aray('start' => 140104, 'stop') ....)
 * @param array $ranges2 (of structure array( array('start' => 1203120, 'stop' => 120399), aray('start' => 140104, 'stop') ....)
 * @return array
 *
 * Assumption: The argument sets may contain overlapping timeranges, which are wrong in principle, but we'll manage anyway.
 */
function merge_timerange_sets(&$timerange_set1, &$timerange_set2)
{
	$resulting_timerange_set = array();

	# plan: add both ranges to third set, merging as we go along

	foreach($timerange_set1 as $range)
	{
		add_timerange_to_set($range, $resulting_timerange_set);
	}

	foreach($timerange_set2 as $range)
	{
		add_timerange_to_set($range, $resulting_timerange_set);
	}
	return $resulting_timerange_set;
}


function is_daterange_single_day(&$dr)
{
	if($dr['syear'] != $dr['eyear'])
		return false;
	if($dr['smon'] != $dr['emon'])
		return false;
	if($dr['smday'] != $dr['emday'])
		return false;
	if($dr['swday'] != $dr['ewday'])
		return false;
	if($dr['swday_offset'] != $dr['ewday_offset'])
		return false;

	return true;
}

function print_timerange(&$r)
{
	print "$r[start]-$r[stop]";
}

function subtract_timerange_sets(&$set_include, &$set_exclude)
{
	for($i=0,$num_includes=count($set_include) ; $i<$num_includes ; $i++)
	{
		for($j=0,$num_excludes=count($set_exclude) ; $j<$num_excludes ; $j++)
		{
			$tmp = subtract_timerange($set_include[$i], $set_exclude[$j]);
			$set_include[$i] = $tmp[0];

			# if range was split into two, add other one at end (outside of loop)
			if(count($tmp) > 1)
			{
				$set_include[count($set_include)] = $tmp[1];
			}
		}
	}
	return $set_include;
}

/**
 * Creates the state mask for a specific state
 *
 * @param boolean $host indicates host or service state
 * @param int $state indicates which state
 *
 */
function state_to_mask($host, $state)
{
	if ($host) {
		return 1 << ($state + 1);
	} else {
		return 1 << ($state + 1 + 4);
	}
}

$mask_to_state_dict = array(
							1 => array(true, HOST_PENDING),
							2 => array(true, HOST_UP),
							4 => array(true, HOST_DOWN),
							8 => array(true, HOST_UNREACHABLE),
							16 => array(false, SERVICE_PENDING),
							32 => array(false, SERVICE_OK),
							64 => array(false, SERVICE_WARNING),
							128 => array(false, SERVICE_CRITICAL),
							256 => array(false, SERVICE_UNKNOWN)
							);

$mask_to_state_str_dict = array(
								1 => "Host Pending",
								2 => "Host Up",
								4 => "Host Down",
								8 => "Host Unreachable",
								16 => "Service Pending",
								32 => "Service Ok",
								64 => "Service Warning",
								128 => "Service Critical",
								256 => "Service Unknown"
								);

function mask_to_states($mask)
{
	global $mask_to_state_dict;

	foreach ($mask_to_state_dict as $state_mask => $type_state) {
		if ($mask & $state_mask)
			$retval[] = $type_state;
	}

	return $retval;
}

function mask_to_states_str($mask)
{
	global $mask_to_state_str_dict;

	foreach ($mask_to_state_str_dict as $state_mask => $type_state) {
		if ($mask & $state_mask)
			$retval[] = $type_state;
	}

	return $retval;
}

function states_to_mask($values)
{
	$tmp_mask = 0;

	foreach ($values as $mask) {
		$tmp_mask |= (int) $mask;
	}

	return $tmp_mask;
}

function mask_to_state_masks($mask)
{
	global $mask_to_state_dict;

	#if ($mask == 0)
	#	$mask = 0xffffffff;

	$retval = array();

	foreach ($mask_to_state_dict as $state_mask => $type_state) {
		if ($mask & $state_mask)
			$retval[] = $state_mask;
	}

	return $retval;
}

?>
