<?php
require_once(dirname(__FILE__).'/../paths.inc.php');
require_once (dirname( __FILE__) . '/../sql_class.php');
require_once (dirname( __FILE__) . '/../lib_reports.php');
require_once (dirname( __FILE__) . '/../lib_gui.php');
global $op5common_path;
require_once($op5common_path . '/lib/common.php');
$sql_class = new sql_class();
$dba = $sql_class->db;
if (!$dba) {
	// db connection failed - bail out
	echo "Database connection failed";
	exit;
}
$map_type_field = array(
	'hosts' => "host_name",
	'services' => "service_description",
	'hostgroups' => "hostgroup",
	'servicegroups' => "servicegroup"
);

$setup_keys = array(
	'sla_name',
	'info',
	'rpttimeperiod',
	'report_period',
	'start_time',
	'end_time',
	'report_type',
	'initialassumedhoststate',
	'initialassumedservicestate',
	'assumeinitialstates',
	'scheduleddowntimeasuptime',
	'assumestatesduringnotrunning',
	'includesoftstates'
);

$sla_field_names = array(
	'hosts' => 'PERCENT_TOTAL_TIME_UP',
	'hostgroups' => 'PERCENT_TOTAL_TIME_UP',
	'services' => 'PERCENT_TOTAL_TIME_OK',
	'servicegroups' => 'PERCENT_TOTAL_TIME_OK'
);

$months = array(
	'Jan',
	'Feb',
	'Mar',
	'Apr',
	'May',
	'Jun',
	'Jul',
	'Aug',
	'Sep',
	'Oct',
	'Nov',
	'Dec'
);

/**
 * Get saved SLA report data from db
 *
 * @return array
 */
function get_saved_slas()
{
	global $dba, $mon_auth;
	if (!$dba) {
		$sql_class = new sql_class();
		$dba = $sql_class->db;
	}

	$sql = "SELECT id, sla_name FROM sla_config ";
	if (!$mon_auth->view_hosts_root) {
		$sql .= "WHERE user='".get_array_var($_SERVER, 'REMOTE_USER')."' OR user=''";
	}
	$sql .= " ORDER BY sla_name";

	$return = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $return;
}

/**
*	@name 	get_sla_info
*	@desc 	Fetch info on single saved SLA report
*
*/
function get_sla_info($id=false)
{
	global $dba;
	if (empty($id)) return false;

	$sql = "SELECT * FROM sla_config WHERE id=".(int)$id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	$return = $res[0];
	$period_info = get_period_info($id);
	if ($period_info) {
		foreach ($period_info as $row) {
			$month_key =  $row['name'];
			if ($return['report_period'] == 'lastmonth') {
				# special case lastmonth report period to work as expected,
				# i.e to use the entered SLA value for every month
				# no matter what month it was scheduled
				$month = date('n');
				$month = $month == 1 ? 12 : ($month-1);
				$month_key = 'month_'.$month;
			}

			$return[$month_key] = $row['value'];
		}
	}
	$object_info = get_sla_config_objects($id);
	$objects = false;
	if ($object_info) {
		foreach ($object_info as $row) {
			$objects[] = $row['name'];
		}
	}
	$return['objects'] = $objects;
	return $return;
}

/**
*	@name 	delete_sla
*	@desc 	Delete info on a saved SLA
* 	@param  int $sla_id
*
*/
function delete_sla($sla_id)
{
	global $dba;
	if (empty($sla_id)) return false;
	$err = false;
	$sql[] = "DELETE FROM sla_periods WHERE sla_id=".$sla_id;
	$sql[] = "DELETE FROM sla_config_objects WHERE sla_id=".$sla_id;
	$sql[] = "DELETE FROM sla_config WHERE id=".$sla_id;
	foreach ($sql as $query) {
		$dba->execute($query);
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			$err++;
		}
	}
	if ($err !== false) {
		return false;
	}
	delete_scheduled_report($sla_id, 'sla');
	return true;
}

/**
*	@name get_sla_config_objects
*	@desc
*
*/
function get_sla_config_objects($sla_id=false)
{
	global $dba;
	if (empty($sla_id)) {
		return false;
	}
	$sql = "SELECT * FROM sla_config_objects WHERE sla_id=".(int)$sla_id;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	return $res;
}

/**
*	@name get_period_info
*	@desc
*
*/
function get_period_info($sla_id=false)
{
	global $dba;
	if (empty($sla_id)) {
		return false;
	}
	$sql = "SELECT * FROM sla_periods WHERE sla_id=".(int)$sla_id." ORDER BY id";
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	return $res;
}

/**
*	@name 	edit_sla_info
*	@desc 	save or update information depending on input sla_id.
* 			Calls related functions to save period- and object info
* 			for SLA object.
*
*/
function edit_sla_info($sla_id=false, $sla_options=false, $months=false, $objects=false)
{

	global $dba;
	$update = false;
	if (empty($sla_options) || empty($months) || empty($objects)) {
		return false;
	}

	// INSERT or UPDATE?
	if ($sla_id) $update = true;
	else
	{
		$sql = "SELECT id FROM sla_config WHERE sla_name = '".$sla_options['sla_name']."'";
		$res = $dba->execute($sql);
		if(count($res) > 0)
		{
			$sla_id = $res[0]["id"];
			$update = true;
		}
	}

	if (!$update) {
		$sql = "INSERT INTO sla_config (user, ".implode(', ', array_keys($sla_options)).") VALUES('".get_array_var($_SERVER, 'REMOTE_USER')."', '".implode('\',\'', array_values($sla_options))."')";
	} else {
		$sql = "UPDATE sla_config SET ";
		foreach ($sla_options as $key => $value) {
			$a_sql[] = $key." = '".$value."' ";
		}
		$sql .= implode(', ', $a_sql);
		$sql .= ", updated = now()";
		$sql .= " WHERE id=".$sla_id;
	}
	#echo $sql;
	$dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	// query OK, continue with other tables
	if (!$update) $sla_id = (int)$dba->getInsertId();

	// Insert/Update sla_periods
	if(!save_period_info($sla_id, $months)) {
		return false;
	}

	// insert/update sla_config_objects
	if (!save_config_objects($sla_id, $objects)) {
		return false;
	}
	return $sla_id;
}

/**
*	@name 	save_config_objects
*	@desc 	save information on what objects are related to this
* 			SLA report - hosts/services/-groups
* 			Stores the name of the objects
*
*/
function save_config_objects($sla_id=false, $objects=false)
{
	global $dba;
	if (empty($objects) || empty($sla_id)) return false;

	// remove old records (if any)
	$sql = "DELETE FROM sla_config_objects WHERE sla_id=".$sla_id;
	$dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	unset($sql);

	$_sql = "INSERT INTO sla_config_objects (sla_id, name) VALUES(";
	foreach ($objects as $item) {
		$query = $_sql.(int)$sla_id.", '".$item."')";
		$dba->execute($query);
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			return false;
		}
	}
	return true;
}

/**
*	@name 	save_period_info
*	@desc 	Handle info on SLA period values (monthly)
* 			Remove old values and enter the new ones
*
*/
function save_period_info($sla_id=false, $months=false)
{
	global $dba;
	// remove old records (if any)
	$sql = "DELETE FROM sla_periods WHERE sla_id=".$sla_id;
	$dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}
	unset($sql);

	if (!empty($months)) {
		foreach ($months as $key => $value) {
			#echo "$key => $value<br />";
			$sql[] = "INSERT INTO sla_periods(sla_id, name, value) VALUES(".(int)$sla_id.", 'month_".$key."', ".floatval($value).")";
		}
	}
	foreach ($sql as $query) {
		$dba->execute($query);
		if (!empty($dba->error)) {
			__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $query);
			return false;
		}
	}
	return true;
}

/**
*	@name 	get_month_values
*	@desc 	Extract submitted SLA values from 'month_' fields
* 			Will only return fields actually set
* 	@param 	array $request
* 	@return array
*
*/
function get_month_values($request=false)
{
	$month_val = false;
	$id = false;
	foreach ($request as $key => $value) {
		if (preg_match('/^month/', $key)) {
			$id = (int)str_replace('month_', '', $key);
			if (trim($value) == '') continue;
			$value = str_replace(',', '.', $value);
			$value = (float)$value;
			// values greater than 100 doesn't make sense
			if ($value>100)
				continue;
			$month_val[$id] = $value;
		}
	}
	return $month_val;
}

/**
*	@name 	get_sla_data
*	@desc 	Fetch data from report_class
* 			Uses split_month_data() to split start- and end_time
* 			on months.
*/
function get_sla_data($sla_options=false, $months=false, $objects=false)
{
	global $options, $dep_vars, $_REQUEST, $err_msg;
$options = array(
	'rpttimeperiod' => 'report_timeperiod',
	'scheduleddowntimeasuptime' => 'scheduled_downtime_as_uptime',
	'assumestatesduringnotrunning' => 'assume_states_during_not_running',
	'includesoftstates' => 'include_soft_states',
	'assumeinitialstates' => 'assume_initial_states',
	'use_average' => 'use_average',
	'keep_logs' => 'keep_logs',
	'keep_logs_maxrows' => 'keep_logs_maxrows',
	'keep_logs_states' => 'keep_logs_states'
);
$dep_vars = array(
	'assumeinitialstates' => array(
		'initialassumedhoststate' => 'initial_assumed_host_state',
		'initialassumedservicestate' => 'initial_assumed_service_state'
	)
);

	if (empty($sla_options) || empty($months) || empty($objects)) {
		return false;
	}
	if (is_int($sla_options['start_time']) && is_int($sla_options['end_time'])) {
		$time_parts = array($sla_options['start_time'], $sla_options['end_time']);
	} else {
		$time_parts = calculate_time($sla_options['report_period'], $sla_options['start_time'], $sla_options['end_time']);
	}
	$start_date = $time_parts[0];
	$end_date 	= $time_parts[1];

	// OK, we have start and end but we will have to split
	// this time into parts according to sla_periods (months)
	$time_tmp = split_month_data($months, $start_date, $end_date);
	$time_arr = $time_tmp;//array();
	// only use month entered by the user regardless of start- or endtime
	$option_name = false;
	if (preg_match('/groups$/', trim($sla_options['report_type']))) {
		foreach ($time_arr as $mnr => $dates) {
			$data_tmp = expand_group_request($objects, substr($sla_options['report_type'], 0, strlen($sla_options['report_type'])-1), $dates['start'], $dates['end']);

			if (!empty($data_tmp))
				foreach ($data_tmp as $val) {
					if ($val !== false) {
						#var_dump($val);
					$data[$val['groupname']][$mnr] = array('source' => $val['source'], 'states' => $val['states'], 'tot_time' => $val['tot_time'], 'groupname' => $val['groupname'],
														   'log' => $val['log'], 'log_maxrows' => $val['log_maxrows']
														   );
					}
				}
		}
	} else {
		$option_name = preg_match('/hosts/', $sla_options['report_type']) ? 'host_name' : 'service_description';
		foreach ($time_arr as $mnr => $dates) {
			$report_class = new report_class();
			foreach ($options as $var => $new_var) {
				if (!$report_class->set_option($new_var, get_array_var($_REQUEST, $var))) {
					$err_msg .= "Could not set option '$new_var' to '".convert_yesno_int(get_array_var($_REQUEST, $var))."'<br />";
				}
			}
			foreach ($dep_vars as $check => $set)
			{
				if (isset($_REQUEST[$check]) && !empty($_REQUEST[$check]))
				{
					foreach ($set as $dep => $key)
					{
						if (!$report_class->set_option($key, $_REQUEST[$dep]))
						{
							$err_msg .= "Could not set option '".$key."' to '".$_REQUEST[$dep]."'<br />";
						}
					}
				}
			}
			$report_class->set_option($option_name, $objects);
			$data_tmp = $report_class->get_uptime(false, false, $dates['start'], $dates['end']);

			# THE NEXT LINE EXTRACT _GROUPWIDE STATES_, DISCARDS INDIVIDUAL MEMBER INFO (numeric indices)
			$data[$mnr] = array('source' => $data_tmp['source'], 'states' => $data_tmp['states'], 'tot_time' => $data_tmp['tot_time'], 'groupname' => $data_tmp['groupname'],
								'log' => $data_tmp['log'], 'log_maxrows' => $data_tmp['log_maxrows']
								);
		}
	}
	return $data;
}

/**
*	@name split_month_data
*	@desc  Splits a span of unixtime(start_time, end_time) into slices for every month number in $months.
*  @param $months array - DEPRECATED. the months to calculate for.
*  @param $start_time int start timestamp of the first month
*  @param $end_time int end timestamp of the last month
*  @return array of start/end timestamps for every timestamp gives the start and end of the month
*/
function split_month_data($months=false, $start_time=false, $end_time=false)
{
	if (empty($months) || empty($start_time) || empty($end_time)) {
		return false;
	}
	$date = $start_time;
	while ($date < $end_time) {
		$end = strtotime('+1 month', $date);
		$return[date('n', $date)] = array('start' => $date, 'end' => $end);
		$date = $end;
	}
	return $return;
}

/**
 * Discovers name of a report data object
 *
 * @param array $sla_data
 * @return mixed String name of object if found, false else.
 */
function get_sla_group_name(&$sla_data)
{
	if (empty($sla_data)) return false;

	$first_elem = each($sla_data);
	if(is_numeric($first_elem['key']))
		$sla_entry = current($sla_data) != false ? current($sla_data) : $first_elem['value'];
	else
		$sla_entry =& $sla_data;

	// hostgroup or servicegroup
	if(!empty($sla_entry['groupname']))
		return $sla_entry['groupname'];

	// custom group
	if(strpos($sla_entry['source'], ',') !== false)
	{
		return $sla_entry['source'];
	}

	// single service
	if(get_array_var($sla_entry['states'], 'SERVICE_DESCRIPTION'))
		// concatenate with host since lib_reports return service without that part
		return $sla_entry['states']['HOST_NAME'].';'.$sla_entry['states']['SERVICE_DESCRIPTION'];

	// single host
	return $sla_entry['states']['HOST_NAME'];
}

/**
 * Helper function to print array content.
 *
 * @param array $ary
 */
function print_array($ary)
{
	echo "<pre>";
	print_r($ary);
	echo "</pre>";
}

/**
*	@name 	get_date_ranges
*	@desc 	Fetch info on first and last timestamp in db
*
*/
function get_date_ranges()
{
	global $dba;
	$report_class = new report_class();
	$sql = "
		SELECT
			MIN(timestamp) AS min_date,
			MAX(timestamp) AS max_date
		FROM
			".$report_class->db_name.".".$report_class->db_table;
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	$row = $res[0];
	$min_date = $row['min_date'];
	$max_date = $row['max_date'];
	return array($min_date, $max_date);
}

/**
 * Fetch values for valid months depending on year
 *
 * @param int $the_year
 * @param str $type, 'start', 'end'
 * @param str $item, 'year', 'month'
 */
function print_date_ranges($the_year, $type='start', $item='year')
{
	$objResponse = new xajaxResponse();
	$date_ranges = get_date_ranges();
	if (empty($date_ranges)) return false;
	$start_date 	= $date_ranges[0]; 	// first date in db
	$end_date 		= $date_ranges[1];	// last date in db
	$type 			= trim($type);
	$item 			= trim($item);
	$the_year 		= (int)$the_year;
	$start_year 	= date('Y', $start_date);
	$current_year	= date('Y');
	$current_month	= date('m');
	$end_year 		= date('Y', $end_date);
	$start_month 	= date('m', $start_date);
	$end_month 		= date('m', $end_date);
	$objResponse->call("log", "Current: $current_year, Start year: $start_year, End Year = $end_year, Start month = $start_month");
	if (empty($type)) {
		// Print all years
		for ($i=$start_year;$i<=$end_year;$i++) {
			$objResponse->call("sla_addOption", "start_year", $i, $i);
			$objResponse->call("sla_addOption", "end_year", $i, $i);
		}
	} else {
		// empty month list
		$objResponse->call("empty_list", $type."_month");
		if (!empty($the_year)) {
			// end month should always be 12 unless we only
			// have data for a single year
			if ($start_year == $end_year) {
				$end_num = $end_month;
				$start_num = $start_month;
			} else {
				$end_num = $the_year == $current_year ? $current_month: 12;
				$start_num = $the_year == $start_year ? $start_month : 1;
			}
		}
		for ($i=$start_num;$i<=$end_num;$i++) {
			$objResponse->call("sla_addOption", $type."_".$item, str_pad($i, 2, '0', STR_PAD_LEFT), $i);
		}
	}
	return $objResponse;
}

/**
 * @name 	get_all_sla_info_names
 * @desc   	Fetches all info names from sla_config
 * @return array all names
 *
 */
function get_all_sla_config_names()
{
	global $dba;
	$sql = "SELECT sla_name FROM sla_config";
	$res = $dba->execute($sql);
	if (!empty($dba->error)) {
		__debug(__FUNCTION__ . ": line: " . __LINE__, $dba->error, $sql);
		return false;
	}

	$names = array();
	foreach($res as $row)
	{
		$names[] = $row['sla_name'];
	}
	return $names;
}

/**
 * @name generate_sla_member_link
 * @param string $member
 * @param string $object_varname
 * @param array $sla_options SLA input parameters
 * @param array $in_months the SLA month values
 * @return array Links to SLA report for individual members
 */
function generate_sla_member_link($member, $object_varname, &$sla_options, &$in_months)
{
	global $map_type_field;
	$return = "<a href='sla.php?{$object_varname}[]=$member";
	foreach($sla_options as $key => $val)
	{
		switch ($key)
		{
			case 'report_type':
				$val = array_search($object_varname, $map_type_field);
				if($val === FALSE)  // if custom group
					$val = $object_varname;
				break;
			case 'start_time': case 'end_time':
				if (is_numeric($val)) {
					$val = date('Y-m-d H:i', $val);
				}
			    break;
		    case 'keep_logs_states_mask':
			    foreach (mask_to_state_masks($val) as $mask) {
					$return .= "&keep_logs_states[]=" . $mask;
				}
			    continue;
		}
		$return .= "&$key=$val";
	}
	foreach($in_months as $month => $sla)
	{
		$return .= "&month_$month=$sla";
	}
	$host_alias = '';
	$service_description = '';
	$host_name = '';
	if (array_key_exists('use_alias', $sla_options) && $sla_options['use_alias'] == 1) {
		# use alias with host_name
		if (strstr($member, ';')) {
			# we have host_name;service_description so we neeed to split this
			$member_parts = explode(';', $member);
			if (is_array($member_parts) && sizeof($member_parts)==2) {
				$host_name = $member_parts[0];
				$host_alias = get_host_alias($host_name);
				$service_description = $member_parts[1];
				$member = $service_description . ' on ' . $host_alias.' ('.$host_name.') ';
			}
		} else {
			$host_alias = get_host_alias($member);
			$member = $host_alias.' (' . $member . ')';
		}
	}
	$return .= "'>$member</a>";

	return $return;
}

/**
 * @name 	generate_avail_member_link
 * @param 	string $members
 * @param 	string $object_varname
 * @param 	array $sla_options SLA input parameters
 * @return 	array Links to Availability report for individual members
 */
function generate_avail_member_link($members, $object_varname, &$sla_options)
{
	global $map_type_field;
	$objects = '';
	$return = "../avail_result.php?";
	if (is_array($members)) {
		$objects .= implode('&amp;'.$object_varname.'[]=',$members);
	} else {
		$objects = $members;
	}
	$return .= $object_varname.'[]='.$objects;
	foreach($sla_options as $key => $val)
	{
		switch ($key)
		{
			case 'report_type':
				$val = array_search($object_varname, $map_type_field);
				if($val === FALSE)  // if custom group
					$val = $object_varname;
				break;
			case 'start_time': case 'end_time':
				if (is_numeric($val)) {
					$val = date('Y-m-d H:i', $val);
				}
				break;

		}
		$return .= "&amp;$key=$val";
	}
	return $return;
}

?>
